# -*- coding: utf-8 -*-
"""
Linode Cloud Module using Linode's REST API
===========================================

The Linode cloud module is used to control access to the Linode VPS system.

Use of this module only requires the ``apikey`` parameter. However, the default root password for new instances
also needs to be set. The password needs to be 8 characters and contain lowercase, uppercase, and numbers.

Set up the cloud configuration at ``/etc/salt/cloud.providers`` or ``/etc/salt/cloud.providers.d/linode.conf``:

.. code-block:: yaml

    my-linode-provider:
      apikey: f4ZsmwtB1c7f85Jdu43RgXVDFlNjuJaeIYV8QMftTqKScEB2vSosFSr...
      password: F00barbaz
      driver: linode

    linode-profile:
      provider: my-linode-provider
      size: Linode 1024
      image: CentOS 7
      location: London, England, UK

"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

import datetime
import logging
import pprint
import re
import time

# Import Salt Libs
import salt.config as config
from salt.exceptions import (
    SaltCloudConfigError,
    SaltCloudException,
    SaltCloudNotFound,
    SaltCloudSystemExit,
)
from salt.ext import six
from salt.ext.six.moves import range

# Get logging started
log = logging.getLogger(__name__)

# The epoch of the last time a query was made
LASTCALL = int(time.mktime(datetime.datetime.now().timetuple()))

# Human-readable status fields (documentation: https://www.linode.com/api/linode/linode.list)
LINODE_STATUS = {
    "boot_failed": {"code": -2, "descr": "Boot Failed (not in use)"},
    "beeing_created": {"code": -1, "descr": "Being Created"},
    "brand_new": {"code": 0, "descr": "Brand New"},
    "running": {"code": 1, "descr": "Running"},
    "poweroff": {"code": 2, "descr": "Powered Off"},
    "shutdown": {"code": 3, "descr": "Shutting Down (not in use)"},
    "save_to_disk": {"code": 4, "descr": "Saved to Disk (not in use)"},
}

__virtualname__ = "linode"


# Only load in this module if the Linode configurations are in place
def __virtual__():
    """
    Check for Linode configs.
    """
    if get_configured_provider() is False:
        return False

    return __virtualname__


def get_configured_provider():
    """
    Return the first configured instance.
    """
    return config.is_provider_configured(
        __opts__, __active_provider_name__ or __virtualname__, ("apikey", "password",)
    )


def avail_images(call=None):
    """
    Return available Linode images.

    CLI Example:

    .. code-block:: bash

        salt-cloud --list-images my-linode-config
        salt-cloud -f avail_images my-linode-config
    """
    if call == "action":
        raise SaltCloudException(
            "The avail_images function must be called with -f or --function."
        )

    response = _query("avail", "distributions")

    ret = {}
    for item in response["DATA"]:
        name = item["LABEL"]
        ret[name] = item

    return ret


def avail_locations(call=None):
    """
    Return available Linode datacenter locations.

    CLI Example:

    .. code-block:: bash

        salt-cloud --list-locations my-linode-config
        salt-cloud -f avail_locations my-linode-config
    """
    if call == "action":
        raise SaltCloudException(
            "The avail_locations function must be called with -f or --function."
        )

    response = _query("avail", "datacenters")

    ret = {}
    for item in response["DATA"]:
        name = item["LOCATION"]
        ret[name] = item

    return ret


def avail_sizes(call=None):
    """
    Return available Linode sizes.

    CLI Example:

    .. code-block:: bash

        salt-cloud --list-sizes my-linode-config
        salt-cloud -f avail_sizes my-linode-config
    """
    if call == "action":
        raise SaltCloudException(
            "The avail_locations function must be called with -f or --function."
        )

    response = _query("avail", "LinodePlans")

    ret = {}
    for item in response["DATA"]:
        name = item["LABEL"]
        ret[name] = item

    return ret


def boot(name=None, kwargs=None, call=None):
    """
    Boot a Linode.

    name
        The name of the Linode to boot. Can be used instead of ``linode_id``.

    linode_id
        The ID of the Linode to boot. If provided, will be used as an
        alternative to ``name`` and reduces the number of API calls to
        Linode by one. Will be preferred over ``name``.

    config_id
        The ID of the Config to boot. Required.

    check_running
        Defaults to True. If set to False, overrides the call to check if
        the VM is running before calling the linode.boot API call. Change
        ``check_running`` to True is useful during the boot call in the
        create function, since the new VM will not be running yet.

    Can be called as an action (which requires a name):

    .. code-block:: bash

        salt-cloud -a boot my-instance config_id=10

    ...or as a function (which requires either a name or linode_id):

    .. code-block:: bash

        salt-cloud -f boot my-linode-config name=my-instance config_id=10
        salt-cloud -f boot my-linode-config linode_id=1225876 config_id=10
    """
    if name is None and call == "action":
        raise SaltCloudSystemExit("The boot action requires a 'name'.")

    if kwargs is None:
        kwargs = {}

    linode_id = kwargs.get("linode_id", None)
    config_id = kwargs.get("config_id", None)
    check_running = kwargs.get("check_running", True)

    if call == "function":
        name = kwargs.get("name", None)

    if name is None and linode_id is None:
        raise SaltCloudSystemExit(
            "The boot function requires either a 'name' or a 'linode_id'."
        )

    if config_id is None:
        raise SaltCloudSystemExit("The boot function requires a 'config_id'.")

    if linode_id is None:
        linode_id = get_linode_id_from_name(name)
        linode_item = name
    else:
        linode_item = linode_id

    # Check if Linode is running first
    if check_running is True:
        status = get_linode(kwargs={"linode_id": linode_id})["STATUS"]
        if status == "1":
            raise SaltCloudSystemExit(
                "Cannot boot Linode {0}. "
                "Linode {0} is already running.".format(linode_item)
            )

    # Boot the VM and get the JobID from Linode
    response = _query(
        "linode", "boot", args={"LinodeID": linode_id, "ConfigID": config_id}
    )["DATA"]
    boot_job_id = response["JobID"]

    if not _wait_for_job(linode_id, boot_job_id):
        log.error("Boot failed for Linode %s.", linode_item)
        return False

    return True


def clone(kwargs=None, call=None):
    """
    Clone a Linode.

    linode_id
        The ID of the Linode to clone. Required.

    datacenter_id
        The ID of the Datacenter where the Linode will be placed. Required.

    plan_id
        The ID of the plan (size) of the Linode. Required.

    CLI Example:

    .. code-block:: bash

        salt-cloud -f clone my-linode-config linode_id=1234567 datacenter_id=2 plan_id=5
    """
    if call == "action":
        raise SaltCloudSystemExit(
            "The clone function must be called with -f or --function."
        )

    if kwargs is None:
        kwargs = {}

    linode_id = kwargs.get("linode_id", None)
    datacenter_id = kwargs.get("datacenter_id", None)
    plan_id = kwargs.get("plan_id", None)
    required_params = [linode_id, datacenter_id, plan_id]

    for item in required_params:
        if item is None:
            raise SaltCloudSystemExit(
                "The clone function requires a 'linode_id', 'datacenter_id', "
                "and 'plan_id' to be provided."
            )

    clone_args = {
        "LinodeID": linode_id,
        "DatacenterID": datacenter_id,
        "PlanID": plan_id,
    }

    return _query("linode", "clone", args=clone_args)


def create(vm_):
    """
    Create a single Linode VM.
    """
    name = vm_["name"]
    try:
        # Check for required profile parameters before sending any API calls.
        if (
            vm_["profile"]
            and config.is_profile_configured(
                __opts__, __active_provider_name__ or "linode", vm_["profile"], vm_=vm_
            )
            is False
        ):
            return False
    except AttributeError:
        pass

    if _validate_name(name) is False:
        return False

    __utils__["cloud.fire_event"](
        "event",
        "starting create",
        "salt/cloud/{0}/creating".format(name),
        args=__utils__["cloud.filter_event"](
            "creating", vm_, ["name", "profile", "provider", "driver"]
        ),
        sock_dir=__opts__["sock_dir"],
        transport=__opts__["transport"],
    )

    log.info("Creating Cloud VM %s", name)

    data = {}
    kwargs = {"name": name}

    plan_id = None
    size = vm_.get("size")
    if size:
        kwargs["size"] = size
        plan_id = get_plan_id(kwargs={"label": size})

    datacenter_id = None
    location = vm_.get("location")
    if location:
        try:
            datacenter_id = get_datacenter_id(location)
        except KeyError:
            # Linode's default datacenter is Dallas, but we still have to set one to
            # use the create function from Linode's API. Dallas's datacenter id is 2.
            datacenter_id = 2

    clonefrom_name = vm_.get("clonefrom")
    cloning = True if clonefrom_name else False
    if cloning:
        linode_id = get_linode_id_from_name(clonefrom_name)
        clone_source = get_linode(kwargs={"linode_id": linode_id})

        kwargs = {
            "clonefrom": clonefrom_name,
            "image": "Clone of {0}".format(clonefrom_name),
        }

        if size is None:
            size = clone_source["TOTALRAM"]
            kwargs["size"] = size
            plan_id = clone_source["PLANID"]

        if location is None:
            datacenter_id = clone_source["DATACENTERID"]

        # Create new Linode from cloned Linode
        try:
            result = clone(
                kwargs={
                    "linode_id": linode_id,
                    "datacenter_id": datacenter_id,
                    "plan_id": plan_id,
                }
            )
        except Exception as err:  # pylint: disable=broad-except
            log.error(
                "Error cloning '%s' on Linode.\n\n"
                "The following exception was thrown by Linode when trying to "
                "clone the specified machine:\n%s",
                clonefrom_name,
                err,
                exc_info_on_loglevel=logging.DEBUG,
            )
            return False
    else:
        kwargs["image"] = vm_["image"]

        # Create Linode
        try:
            result = _query(
                "linode",
                "create",
                args={"PLANID": plan_id, "DATACENTERID": datacenter_id},
            )
        except Exception as err:  # pylint: disable=broad-except
            log.error(
                "Error creating %s on Linode\n\n"
                "The following exception was thrown by Linode when trying to "
                "run the initial deployment:\n%s",
                name,
                err,
                exc_info_on_loglevel=logging.DEBUG,
            )
            return False

    if "ERRORARRAY" in result:
        for error_data in result["ERRORARRAY"]:
            log.error(
                "Error creating %s on Linode\n\n"
                "The Linode API returned the following: %s\n",
                name,
                error_data["ERRORMESSAGE"],
            )
            return False

    __utils__["cloud.fire_event"](
        "event",
        "requesting instance",
        "salt/cloud/{0}/requesting".format(name),
        args=__utils__["cloud.filter_event"](
            "requesting", vm_, ["name", "profile", "provider", "driver"]
        ),
        sock_dir=__opts__["sock_dir"],
        transport=__opts__["transport"],
    )

    node_id = _clean_data(result)["LinodeID"]
    data["id"] = node_id

    if not _wait_for_status(node_id, status=(_get_status_id_by_name("brand_new"))):
        log.error(
            "Error creating %s on LINODE\n\n" "while waiting for initial ready status",
            name,
            exc_info_on_loglevel=logging.DEBUG,
        )

    # Update the Linode's Label to reflect the given VM name
    update_linode(node_id, update_args={"Label": name})
    log.debug("Set name for %s - was linode%s.", name, node_id)

    # Add private IP address if requested
    private_ip_assignment = get_private_ip(vm_)
    if private_ip_assignment:
        create_private_ip(node_id)

    # Define which ssh_interface to use
    ssh_interface = _get_ssh_interface(vm_)

    # If ssh_interface is set to use private_ips, but assign_private_ip
    # wasn't set to True, let's help out and create a private ip.
    if ssh_interface == "private_ips" and private_ip_assignment is False:
        create_private_ip(node_id)
        private_ip_assignment = True

    if cloning:
        config_id = get_config_id(kwargs={"linode_id": node_id})["config_id"]
    else:
        # Create disks and get ids
        log.debug("Creating disks for %s", name)
        root_disk_id = create_disk_from_distro(vm_, node_id)["DiskID"]
        swap_disk_id = create_swap_disk(vm_, node_id)["DiskID"]

        # Create a ConfigID using disk ids
        config_id = create_config(
            kwargs={
                "name": name,
                "linode_id": node_id,
                "root_disk_id": root_disk_id,
                "swap_disk_id": swap_disk_id,
            }
        )["ConfigID"]

    # Boot the Linode
    boot(kwargs={"linode_id": node_id, "config_id": config_id, "check_running": False})

    node_data = get_linode(kwargs={"linode_id": node_id})
    ips = get_ips(node_id)
    state = int(node_data["STATUS"])

    data["image"] = kwargs["image"]
    data["name"] = name
    data["size"] = size
    data["state"] = _get_status_descr_by_id(state)
    data["private_ips"] = ips["private_ips"]
    data["public_ips"] = ips["public_ips"]

    # Pass the correct IP address to the bootstrap ssh_host key
    if ssh_interface == "private_ips":
        vm_["ssh_host"] = data["private_ips"][0]
    else:
        vm_["ssh_host"] = data["public_ips"][0]

    # If a password wasn't supplied in the profile or provider config, set it now.
    vm_["password"] = get_password(vm_)

    # Make public_ips and private_ips available to the bootstrap script.
    vm_["public_ips"] = ips["public_ips"]
    vm_["private_ips"] = ips["private_ips"]

    # Send event that the instance has booted.
    __utils__["cloud.fire_event"](
        "event",
        "waiting for ssh",
        "salt/cloud/{0}/waiting_for_ssh".format(name),
        sock_dir=__opts__["sock_dir"],
        args={"ip_address": vm_["ssh_host"]},
        transport=__opts__["transport"],
    )

    # Bootstrap!
    ret = __utils__["cloud.bootstrap"](vm_, __opts__)

    ret.update(data)

    log.info("Created Cloud VM '%s'", name)
    log.debug("'%s' VM creation details:\n%s", name, pprint.pformat(data))

    __utils__["cloud.fire_event"](
        "event",
        "created instance",
        "salt/cloud/{0}/created".format(name),
        args=__utils__["cloud.filter_event"](
            "created", vm_, ["name", "profile", "provider", "driver"]
        ),
        sock_dir=__opts__["sock_dir"],
        transport=__opts__["transport"],
    )

    return ret


def create_config(kwargs=None, call=None):
    """
    Creates a Linode Configuration Profile.

    name
        The name of the VM to create the config for.

    linode_id
        The ID of the Linode to create the configuration for.

    root_disk_id
        The Root Disk ID to be used for this config.

    swap_disk_id
        The Swap Disk ID to be used for this config.

    data_disk_id
        The Data Disk ID to be used for this config.

    .. versionadded:: 2016.3.0

    kernel_id
        The ID of the kernel to use for this configuration profile.
    """
    if call == "action":
        raise SaltCloudSystemExit(
            "The create_config function must be called with -f or --function."
        )

    if kwargs is None:
        kwargs = {}

    name = kwargs.get("name", None)
    linode_id = kwargs.get("linode_id", None)
    root_disk_id = kwargs.get("root_disk_id", None)
    swap_disk_id = kwargs.get("swap_disk_id", None)
    data_disk_id = kwargs.get("data_disk_id", None)
    kernel_id = kwargs.get("kernel_id", None)

    if kernel_id is None:
        # 138 appears to always be the latest 64-bit kernel for Linux
        kernel_id = 138

    required_params = [name, linode_id, root_disk_id, swap_disk_id]
    for item in required_params:
        if item is None:
            raise SaltCloudSystemExit(
                "The create_config functions requires a 'name', 'linode_id', "
                "'root_disk_id', and 'swap_disk_id'."
            )

    disklist = "{0},{1}".format(root_disk_id, swap_disk_id)
    if data_disk_id is not None:
        disklist = "{0},{1},{2}".format(root_disk_id, swap_disk_id, data_disk_id)

    config_args = {
        "LinodeID": linode_id,
        "KernelID": kernel_id,
        "Label": name,
        "DiskList": disklist,
    }

    result = _query("linode", "config.create", args=config_args)

    return _clean_data(result)


def create_disk_from_distro(vm_, linode_id, swap_size=None):
    r"""
    Creates the disk for the Linode from the distribution.

    vm\_
        The VM profile to create the disk for.

    linode_id
        The ID of the Linode to create the distribution disk for. Required.

    swap_size
        The size of the disk, in MB.

    """
    kwargs = {}

    if swap_size is None:
        swap_size = get_swap_size(vm_)

    pub_key = get_pub_key(vm_)
    root_password = get_password(vm_)

    if pub_key:
        kwargs.update({"rootSSHKey": pub_key})
    if root_password:
        kwargs.update({"rootPass": root_password})
    else:
        raise SaltCloudConfigError("The Linode driver requires a password.")

    kwargs.update(
        {
            "LinodeID": linode_id,
            "DistributionID": get_distribution_id(vm_),
            "Label": vm_["name"],
            "Size": get_disk_size(vm_, swap_size, linode_id),
        }
    )

    result = _query("linode", "disk.createfromdistribution", args=kwargs)

    return _clean_data(result)


def create_swap_disk(vm_, linode_id, swap_size=None):
    r"""
    Creates the disk for the specified Linode.

    vm\_
        The VM profile to create the swap disk for.

    linode_id
        The ID of the Linode to create the swap disk for.

    swap_size
        The size of the disk, in MB.
    """
    kwargs = {}

    if not swap_size:
        swap_size = get_swap_size(vm_)

    kwargs.update(
        {"LinodeID": linode_id, "Label": vm_["name"], "Type": "swap", "Size": swap_size}
    )

    result = _query("linode", "disk.create", args=kwargs)

    return _clean_data(result)


def create_data_disk(vm_=None, linode_id=None, data_size=None):
    r"""
    Create a data disk for the linode (type is hardcoded to ext4 at the moment)

    .. versionadded:: 2016.3.0

    vm\_
        The VM profile to create the data disk for.

    linode_id
        The ID of the Linode to create the data disk for.

    data_size
        The size of the disk, in MB.

    """
    kwargs = {}

    kwargs.update(
        {
            "LinodeID": linode_id,
            "Label": vm_["name"] + "_data",
            "Type": "ext4",
            "Size": data_size,
        }
    )

    result = _query("linode", "disk.create", args=kwargs)
    return _clean_data(result)


def create_private_ip(linode_id):
    r"""
    Creates a private IP for the specified Linode.

    linode_id
        The ID of the Linode to create the IP address for.
    """
    kwargs = {"LinodeID": linode_id}
    result = _query("linode", "ip.addprivate", args=kwargs)

    return _clean_data(result)


def destroy(name, call=None):
    """
    Destroys a Linode by name.

    name
        The name of VM to be be destroyed.

    CLI Example:

    .. code-block:: bash

        salt-cloud -d vm_name
    """
    if call == "function":
        raise SaltCloudException(
            "The destroy action must be called with -d, --destroy, " "-a or --action."
        )

    __utils__["cloud.fire_event"](
        "event",
        "destroying instance",
        "salt/cloud/{0}/destroying".format(name),
        args={"name": name},
        sock_dir=__opts__["sock_dir"],
        transport=__opts__["transport"],
    )

    linode_id = get_linode_id_from_name(name)

    response = _query(
        "linode", "delete", args={"LinodeID": linode_id, "skipChecks": True}
    )

    __utils__["cloud.fire_event"](
        "event",
        "destroyed instance",
        "salt/cloud/{0}/destroyed".format(name),
        args={"name": name},
        sock_dir=__opts__["sock_dir"],
        transport=__opts__["transport"],
    )

    if __opts__.get("update_cachedir", False) is True:
        __utils__["cloud.delete_minion_cachedir"](
            name, __active_provider_name__.split(":")[0], __opts__
        )

    return response


def get_config_id(kwargs=None, call=None):
    """
    Returns a config_id for a given linode.

    .. versionadded:: 2015.8.0

    name
        The name of the Linode for which to get the config_id. Can be used instead
        of ``linode_id``.h

    linode_id
        The ID of the Linode for which to get the config_id. Can be used instead
        of ``name``.

    CLI Example:

    .. code-block:: bash

        salt-cloud -f get_config_id my-linode-config name=my-linode
        salt-cloud -f get_config_id my-linode-config linode_id=1234567
    """
    if call == "action":
        raise SaltCloudException(
            "The get_config_id function must be called with -f or --function."
        )

    if kwargs is None:
        kwargs = {}

    name = kwargs.get("name", None)
    linode_id = kwargs.get("linode_id", None)
    if name is None and linode_id is None:
        raise SaltCloudSystemExit(
            "The get_config_id function requires either a 'name' or a 'linode_id' "
            "to be provided."
        )
    if linode_id is None:
        linode_id = get_linode_id_from_name(name)

    response = _query("linode", "config.list", args={"LinodeID": linode_id})["DATA"]
    config_id = {"config_id": response[0]["ConfigID"]}

    return config_id


def get_datacenter_id(location):
    """
    Returns the Linode Datacenter ID.

    location
        The location, or name, of the datacenter to get the ID from.
    """

    return avail_locations()[location]["DATACENTERID"]


def get_disk_size(vm_, swap, linode_id):
    r"""
    Returns the size of of the root disk in MB.

    vm\_
        The VM to get the disk size for.
    """
    disk_size = get_linode(kwargs={"linode_id": linode_id})["TOTALHD"]
    return config.get_cloud_config_value(
        "disk_size", vm_, __opts__, default=disk_size - swap
    )


def get_data_disk_size(vm_, swap, linode_id):
    """
    Return the size of of the data disk in MB

    .. versionadded:: 2016.3.0
    """
    disk_size = get_linode(kwargs={"linode_id": linode_id})["TOTALHD"]
    root_disk_size = config.get_cloud_config_value(
        "disk_size", vm_, __opts__, default=disk_size - swap
    )
    return disk_size - root_disk_size - swap


def get_distribution_id(vm_):
    r"""
    Returns the distribution ID for a VM

    vm\_
        The VM to get the distribution ID for
    """
    distributions = _query("avail", "distributions")["DATA"]
    vm_image_name = config.get_cloud_config_value("image", vm_, __opts__)

    distro_id = ""

    for distro in distributions:
        if vm_image_name == distro["LABEL"]:
            distro_id = distro["DISTRIBUTIONID"]
            return distro_id

    if not distro_id:
        raise SaltCloudNotFound(
            "The DistributionID for the '{0}' profile could not be found.\n"
            "The '{1}' instance could not be provisioned. The following distributions "
            "are available:\n{2}".format(
                vm_image_name,
                vm_["name"],
                pprint.pprint(
                    sorted(
                        [
                            distro["LABEL"].encode(__salt_system_encoding__)
                            for distro in distributions
                        ]
                    )
                ),
            )
        )


def get_ips(linode_id=None):
    """
    Returns public and private IP addresses.

    linode_id
        Limits the IP addresses returned to the specified Linode ID.
    """
    if linode_id:
        ips = _query("linode", "ip.list", args={"LinodeID": linode_id})
    else:
        ips = _query("linode", "ip.list")

    ips = ips["DATA"]
    ret = {}

    for item in ips:
        node_id = six.text_type(item["LINODEID"])
        if item["ISPUBLIC"] == 1:
            key = "public_ips"
        else:
            key = "private_ips"

        if ret.get(node_id) is None:
            ret.update({node_id: {"public_ips": [], "private_ips": []}})
        ret[node_id][key].append(item["IPADDRESS"])

    # If linode_id was specified, only return the ips, and not the
    # dictionary based on the linode ID as a key.
    if linode_id:
        _all_ips = {"public_ips": [], "private_ips": []}
        matching_id = ret.get(six.text_type(linode_id))
        if matching_id:
            _all_ips["private_ips"] = matching_id["private_ips"]
            _all_ips["public_ips"] = matching_id["public_ips"]

        ret = _all_ips

    return ret


def get_linode(kwargs=None, call=None):
    """
    Returns data for a single named Linode.

    name
        The name of the Linode for which to get data. Can be used instead
        ``linode_id``. Note this will induce an additional API call
        compared to using ``linode_id``.

    linode_id
        The ID of the Linode for which to get data. Can be used instead of
        ``name``.

    CLI Example:

    .. code-block:: bash

        salt-cloud -f get_linode my-linode-config name=my-instance
        salt-cloud -f get_linode my-linode-config linode_id=1234567
    """
    if call == "action":
        raise SaltCloudSystemExit(
            "The get_linode function must be called with -f or --function."
        )

    if kwargs is None:
        kwargs = {}

    name = kwargs.get("name", None)
    linode_id = kwargs.get("linode_id", None)
    if name is None and linode_id is None:
        raise SaltCloudSystemExit(
            "The get_linode function requires either a 'name' or a 'linode_id'."
        )

    if linode_id is None:
        linode_id = get_linode_id_from_name(name)

    result = _query("linode", "list", args={"LinodeID": linode_id})

    return result["DATA"][0]


def get_linode_id_from_name(name):
    """
    Returns the Linode ID for a VM from the provided name.

    name
        The name of the Linode from which to get the Linode ID. Required.
    """
    nodes = _query("linode", "list")["DATA"]

    linode_id = ""
    for node in nodes:
        if name == node["LABEL"]:
            linode_id = node["LINODEID"]
            return linode_id

    if not linode_id:
        raise SaltCloudNotFound(
            "The specified name, {0}, could not be found.".format(name)
        )


def get_password(vm_):
    r"""
    Return the password to use for a VM.

    vm\_
        The configuration to obtain the password from.
    """
    return config.get_cloud_config_value(
        "password",
        vm_,
        __opts__,
        default=config.get_cloud_config_value(
            "passwd", vm_, __opts__, search_global=False
        ),
        search_global=False,
    )


def _decode_linode_plan_label(label):
    """
    Attempts to decode a user-supplied Linode plan label
    into the format in Linode API output

    label
        The label, or name, of the plan to decode.

    Example:
        `Linode 2048` will decode to `Linode 2GB`
    """
    sizes = avail_sizes()

    if label not in sizes:
        if "GB" in label:
            raise SaltCloudException(
                "Invalid Linode plan ({}) specified - call avail_sizes() for all available options".format(
                    label
                )
            )
        else:
            plan = label.split()

            if len(plan) != 2:
                raise SaltCloudException(
                    "Invalid Linode plan ({}) specified - call avail_sizes() for all available options".format(
                        label
                    )
                )

            plan_type = plan[0]
            try:
                plan_size = int(plan[1])
            except TypeError:
                plan_size = 0
                log.debug(
                    "Failed to decode Linode plan label in Cloud Profile: %s", label
                )

            if plan_type == "Linode" and plan_size == 1024:
                plan_type = "Nanode"

            plan_size = plan_size / 1024
            new_label = "{} {}GB".format(plan_type, plan_size)

            if new_label not in sizes:
                raise SaltCloudException(
                    "Invalid Linode plan ({}) specified - call avail_sizes() for all available options".format(
                        new_label
                    )
                )

            log.warning(
                "An outdated Linode plan label was detected in your Cloud "
                "Profile (%s). Please update the profile to use the new "
                "label format (%s) for the requested Linode plan size.",
                label,
                new_label,
            )

            label = new_label

    return sizes[label]["PLANID"]


def get_plan_id(kwargs=None, call=None):
    """
    Returns the Linode Plan ID.

    label
        The label, or name, of the plan to get the ID from.

    CLI Example:

    .. code-block:: bash

        salt-cloud -f get_plan_id linode label="Linode 1024"
    """
    if call == "action":
        raise SaltCloudException(
            "The show_instance action must be called with -f or --function."
        )

    if kwargs is None:
        kwargs = {}

    label = kwargs.get("label", None)
    if label is None:
        raise SaltCloudException("The get_plan_id function requires a 'label'.")

    label = _decode_linode_plan_label(label)

    return label


def get_private_ip(vm_):
    """
    Return True if a private ip address is requested
    """
    return config.get_cloud_config_value(
        "assign_private_ip", vm_, __opts__, default=False
    )


def get_data_disk(vm_):
    """
    Return True if a data disk is requested

    .. versionadded:: 2016.3.0
    """
    return config.get_cloud_config_value(
        "allocate_data_disk", vm_, __opts__, default=False
    )


def get_pub_key(vm_):
    r"""
    Return the SSH pubkey.

    vm\_
        The configuration to obtain the public key from.
    """
    return config.get_cloud_config_value(
        "ssh_pubkey", vm_, __opts__, search_global=False
    )


def get_swap_size(vm_):
    r"""
    Returns the amount of swap space to be used in MB.

    vm\_
        The VM profile to obtain the swap size from.
    """
    return config.get_cloud_config_value("swap", vm_, __opts__, default=128)


def get_vm_size(vm_):
    r"""
    Returns the VM's size.

    vm\_
        The VM to get the size for.
    """
    vm_size = config.get_cloud_config_value("size", vm_, __opts__)
    ram = avail_sizes()[vm_size]["RAM"]

    if vm_size.startswith("Linode"):
        vm_size = vm_size.replace("Linode ", "")

    if ram == int(vm_size):
        return ram
    else:
        raise SaltCloudNotFound(
            "The specified size, {0}, could not be found.".format(vm_size)
        )


def list_nodes(call=None):
    """
    Returns a list of linodes, keeping only a brief listing.

    CLI Example:

    .. code-block:: bash

        salt-cloud -Q
        salt-cloud --query
        salt-cloud -f list_nodes my-linode-config

    .. note::

        The ``image`` label only displays information about the VM's distribution vendor,
        such as "Debian" or "RHEL" and does not display the actual image name. This is
        due to a limitation of the Linode API.
    """
    if call == "action":
        raise SaltCloudException(
            "The list_nodes function must be called with -f or --function."
        )
    return _list_linodes(full=False)


def list_nodes_full(call=None):
    """
    List linodes, with all available information.

    CLI Example:

    .. code-block:: bash

        salt-cloud -F
        salt-cloud --full-query
        salt-cloud -f list_nodes_full my-linode-config

    .. note::

        The ``image`` label only displays information about the VM's distribution vendor,
        such as "Debian" or "RHEL" and does not display the actual image name. This is
        due to a limitation of the Linode API.
    """
    if call == "action":
        raise SaltCloudException(
            "The list_nodes_full function must be called with -f or --function."
        )
    return _list_linodes(full=True)


def list_nodes_min(call=None):
    """
    Return a list of the VMs that are on the provider. Only a list of VM names and
    their state is returned. This is the minimum amount of information needed to
    check for existing VMs.

    .. versionadded:: 2015.8.0

    CLI Example:

    .. code-block:: bash

        salt-cloud -f list_nodes_min my-linode-config
        salt-cloud --function list_nodes_min my-linode-config
    """
    if call == "action":
        raise SaltCloudSystemExit(
            "The list_nodes_min function must be called with -f or --function."
        )

    ret = {}
    nodes = _query("linode", "list")["DATA"]

    for node in nodes:
        name = node["LABEL"]
        this_node = {
            "id": six.text_type(node["LINODEID"]),
            "state": _get_status_descr_by_id(int(node["STATUS"])),
        }

        ret[name] = this_node

    return ret


def list_nodes_select(call=None):
    """
    Return a list of the VMs that are on the provider, with select fields.
    """
    return __utils__["cloud.list_nodes_select"](
        list_nodes_full(), __opts__["query.selection"], call,
    )


def reboot(name, call=None):
    """
    Reboot a linode.

    .. versionadded:: 2015.8.0

    name
        The name of the VM to reboot.

    CLI Example:

    .. code-block:: bash

        salt-cloud -a reboot vm_name
    """
    if call != "action":
        raise SaltCloudException(
            "The show_instance action must be called with -a or --action."
        )

    node_id = get_linode_id_from_name(name)
    response = _query("linode", "reboot", args={"LinodeID": node_id})
    data = _clean_data(response)
    reboot_jid = data["JobID"]

    if not _wait_for_job(node_id, reboot_jid):
        log.error("Reboot failed for %s.", name)
        return False

    return data


def show_instance(name, call=None):
    """
    Displays details about a particular Linode VM. Either a name or a linode_id must
    be provided.

    .. versionadded:: 2015.8.0

    name
        The name of the VM for which to display details.

    CLI Example:

    .. code-block:: bash

        salt-cloud -a show_instance vm_name

    .. note::

        The ``image`` label only displays information about the VM's distribution vendor,
        such as "Debian" or "RHEL" and does not display the actual image name. This is
        due to a limitation of the Linode API.
    """
    if call != "action":
        raise SaltCloudException(
            "The show_instance action must be called with -a or --action."
        )

    node_id = get_linode_id_from_name(name)
    node_data = get_linode(kwargs={"linode_id": node_id})
    ips = get_ips(node_id)
    state = int(node_data["STATUS"])

    ret = {
        "id": node_data["LINODEID"],
        "image": node_data["DISTRIBUTIONVENDOR"],
        "name": node_data["LABEL"],
        "size": node_data["TOTALRAM"],
        "state": _get_status_descr_by_id(state),
        "private_ips": ips["private_ips"],
        "public_ips": ips["public_ips"],
    }

    return ret


def show_pricing(kwargs=None, call=None):
    """
    Show pricing for a particular profile. This is only an estimate, based on
    unofficial pricing sources.

    .. versionadded:: 2015.8.0

    CLI Example:

    .. code-block:: bash

        salt-cloud -f show_pricing my-linode-config profile=my-linode-profile
    """
    if call != "function":
        raise SaltCloudException(
            "The show_instance action must be called with -f or --function."
        )

    profile = __opts__["profiles"].get(kwargs["profile"], {})
    if not profile:
        raise SaltCloudNotFound("The requested profile was not found.")

    # Make sure the profile belongs to Linode
    provider = profile.get("provider", "0:0")
    comps = provider.split(":")
    if len(comps) < 2 or comps[1] != "linode":
        raise SaltCloudException("The requested profile does not belong to Linode.")

    plan_id = get_plan_id(kwargs={"label": profile["size"]})
    response = _query("avail", "linodeplans", args={"PlanID": plan_id})["DATA"][0]

    ret = {}
    ret["per_hour"] = response["HOURLY"]
    ret["per_day"] = ret["per_hour"] * 24
    ret["per_week"] = ret["per_day"] * 7
    ret["per_month"] = response["PRICE"]
    ret["per_year"] = ret["per_month"] * 12

    return {profile["profile"]: ret}


def start(name, call=None):
    """
    Start a VM in Linode.

    name
        The name of the VM to start.

    CLI Example:

    .. code-block:: bash

        salt-cloud -a stop vm_name
    """
    if call != "action":
        raise SaltCloudException("The start action must be called with -a or --action.")

    node_id = get_linode_id_from_name(name)
    node = get_linode(kwargs={"linode_id": node_id})

    if node["STATUS"] == 1:
        return {
            "success": True,
            "action": "start",
            "state": "Running",
            "msg": "Machine already running",
        }

    response = _query("linode", "boot", args={"LinodeID": node_id})["DATA"]

    if _wait_for_job(node_id, response["JobID"]):
        return {"state": "Running", "action": "start", "success": True}
    else:
        return {"action": "start", "success": False}


def stop(name, call=None):
    """
    Stop a VM in Linode.

    name
        The name of the VM to stop.

    CLI Example:

    .. code-block:: bash

        salt-cloud -a stop vm_name
    """
    if call != "action":
        raise SaltCloudException("The stop action must be called with -a or --action.")

    node_id = get_linode_id_from_name(name)
    node = get_linode(kwargs={"linode_id": node_id})

    if node["STATUS"] == 2:
        return {"success": True, "state": "Stopped", "msg": "Machine already stopped"}

    response = _query("linode", "shutdown", args={"LinodeID": node_id})["DATA"]

    if _wait_for_job(node_id, response["JobID"]):
        return {"state": "Stopped", "action": "stop", "success": True}
    else:
        return {"action": "stop", "success": False}


def update_linode(linode_id, update_args=None):
    """
    Updates a Linode's properties.

    linode_id
        The ID of the Linode to shutdown. Required.

    update_args
        The args to update the Linode with. Must be in dictionary form.
    """
    update_args.update({"LinodeID": linode_id})

    result = _query("linode", "update", args=update_args)

    return _clean_data(result)


def _clean_data(api_response):
    """
    Returns the DATA response from a Linode API query as a single pre-formatted dictionary

    api_response
        The query to be cleaned.
    """
    data = {}
    data.update(api_response["DATA"])

    if not data:
        response_data = api_response["DATA"]
        data.update(response_data)

    return data


def _list_linodes(full=False):
    """
    Helper function to format and parse linode data
    """
    nodes = _query("linode", "list")["DATA"]
    ips = get_ips()

    ret = {}
    for node in nodes:
        this_node = {}
        linode_id = six.text_type(node["LINODEID"])

        this_node["id"] = linode_id
        this_node["image"] = node["DISTRIBUTIONVENDOR"]
        this_node["name"] = node["LABEL"]
        this_node["size"] = node["TOTALRAM"]

        state = int(node["STATUS"])
        this_node["state"] = _get_status_descr_by_id(state)

        for key, val in six.iteritems(ips):
            if key == linode_id:
                this_node["private_ips"] = val["private_ips"]
                this_node["public_ips"] = val["public_ips"]

        if full:
            this_node["extra"] = node

        ret[node["LABEL"]] = this_node

    return ret


def _query(
    action=None,
    command=None,
    args=None,
    method="GET",
    header_dict=None,
    data=None,
    url="https://api.linode.com/",
):
    """
    Make a web call to the Linode API.
    """
    global LASTCALL
    vm_ = get_configured_provider()

    ratelimit_sleep = config.get_cloud_config_value(
        "ratelimit_sleep", vm_, __opts__, search_global=False, default=0,
    )
    apikey = config.get_cloud_config_value("apikey", vm_, __opts__, search_global=False)

    if not isinstance(args, dict):
        args = {}

    if "api_key" not in args.keys():
        args["api_key"] = apikey

    if action and "api_action" not in args.keys():
        args["api_action"] = "{0}.{1}".format(action, command)

    if header_dict is None:
        header_dict = {}

    if method != "POST":
        header_dict["Accept"] = "application/json"

    decode = True
    if method == "DELETE":
        decode = False

    now = int(time.mktime(datetime.datetime.now().timetuple()))
    if LASTCALL >= now:
        time.sleep(ratelimit_sleep)

    result = __utils__["http.query"](
        url,
        method,
        params=args,
        data=data,
        header_dict=header_dict,
        decode=decode,
        decode_type="json",
        text=True,
        status=True,
        hide_fields=["api_key", "rootPass"],
        opts=__opts__,
    )

    if "ERRORARRAY" in result["dict"]:
        if result["dict"]["ERRORARRAY"]:
            error_list = []

            for error in result["dict"]["ERRORARRAY"]:
                msg = error["ERRORMESSAGE"]

                if msg == "Authentication failed":
                    raise SaltCloudSystemExit("Linode API Key is expired or invalid")
                else:
                    error_list.append(msg)
            raise SaltCloudException(
                "Linode API reported error(s): {}".format(", ".join(error_list))
            )

    LASTCALL = int(time.mktime(datetime.datetime.now().timetuple()))
    log.debug("Linode Response Status Code: %s", result["status"])

    return result["dict"]


def _wait_for_job(linode_id, job_id, timeout=300, quiet=True):
    """
    Wait for a Job to return.

    linode_id
        The ID of the Linode to wait on. Required.

    job_id
        The ID of the job to wait for.

    timeout
        The amount of time to wait for a status to update.

    quiet
        Log status updates to debug logs when True. Otherwise, logs to info.
    """
    interval = 5
    iterations = int(timeout / interval)

    for i in range(0, iterations):
        jobs_result = _query("linode", "job.list", args={"LinodeID": linode_id})["DATA"]
        if jobs_result[0]["JOBID"] == job_id and jobs_result[0]["HOST_SUCCESS"] == 1:
            return True

        time.sleep(interval)
        log.log(
            logging.INFO if not quiet else logging.DEBUG,
            "Still waiting on Job %s for Linode %s.",
            job_id,
            linode_id,
        )
    return False


def _wait_for_status(linode_id, status=None, timeout=300, quiet=True):
    """
    Wait for a certain status from Linode.

    linode_id
        The ID of the Linode to wait on. Required.

    status
        The status to look for to update.

    timeout
        The amount of time to wait for a status to update.

    quiet
        Log status updates to debug logs when False. Otherwise, logs to info.
    """
    if status is None:
        status = _get_status_id_by_name("brand_new")

    status_desc_waiting = _get_status_descr_by_id(status)

    interval = 5
    iterations = int(timeout / interval)

    for i in range(0, iterations):
        result = get_linode(kwargs={"linode_id": linode_id})

        if result["STATUS"] == status:
            return True

        status_desc_result = _get_status_descr_by_id(result["STATUS"])

        time.sleep(interval)
        log.log(
            logging.INFO if not quiet else logging.DEBUG,
            "Status for Linode %s is '%s', waiting for '%s'.",
            linode_id,
            status_desc_result,
            status_desc_waiting,
        )

    return False


def _get_status_descr_by_id(status_id):
    """
    Return linode status by ID

    status_id
        linode VM status ID
    """
    for status_name, status_data in six.iteritems(LINODE_STATUS):
        if status_data["code"] == int(status_id):
            return status_data["descr"]
    return LINODE_STATUS.get(status_id, None)


def _get_status_id_by_name(status_name):
    """
    Return linode status description by internalstatus name

    status_name
        internal linode VM status name
    """
    return LINODE_STATUS.get(status_name, {}).get("code", None)


def _validate_name(name):
    """
    Checks if the provided name fits Linode's labeling parameters.

    .. versionadded:: 2015.5.6

    name
        The VM name to validate
    """
    name = six.text_type(name)
    name_length = len(name)
    regex = re.compile(r"^[a-zA-Z0-9][A-Za-z0-9_-]*[a-zA-Z0-9]$")

    if name_length < 3 or name_length > 48:
        ret = False
    elif not re.match(regex, name):
        ret = False
    else:
        ret = True

    if ret is False:
        log.warning(
            "A Linode label may only contain ASCII letters or numbers, dashes, and "
            "underscores, must begin and end with letters or numbers, and be at least "
            "three characters in length."
        )

    return ret


def _get_ssh_interface(vm_):
    """
    Return the ssh_interface type to connect to. Either 'public_ips' (default)
    or 'private_ips'.
    """
    return config.get_cloud_config_value(
        "ssh_interface", vm_, __opts__, default="public_ips", search_global=False
    )



# coding: utf-8

# Python libs
from __future__ import absolute_import

from collections import namedtuple

# Salt libs
import salt.beacons.service as service_beacon
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch

# Salt testing libs
from tests.support.unit import TestCase

PATCH_OPTS = dict(autospec=True, spec_set=True)

FakeProcess = namedtuple("Process", "cmdline pid")


class ServiceBeaconTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test case for salt.beacons.service
    """

    def setup_loader_modules(self):
        return {service_beacon: {"__context__": {}, "__salt__": {}}}

    def test_non_list_config(self):
        config = {}

        ret = service_beacon.validate(config)

        self.assertEqual(
            ret, (False, "Configuration for service beacon must be a list.")
        )

    def test_empty_config(self):
        config = [{}]

        ret = service_beacon.validate(config)

        self.assertEqual(
            ret, (False, "Configuration for service beacon requires services.")
        )

    def test_service_running(self):
        with patch.dict(
            service_beacon.__salt__, {"service.status": MagicMock(return_value=True)}
        ):
            config = [{"services": {"salt-master": {}}}]

            ret = service_beacon.validate(config)

            self.assertEqual(ret, (True, "Valid beacon configuration"))

            ret = service_beacon.beacon(config)
            self.assertEqual(
                ret,
                [
                    {
                        "service_name": "salt-master",
                        "tag": "salt-master",
                        "salt-master": {"running": True},
                    }
                ],
            )

    def test_service_not_running(self):
        with patch.dict(
            service_beacon.__salt__, {"service.status": MagicMock(return_value=False)}
        ):
            config = [{"services": {"salt-master": {}}}]

            ret = service_beacon.validate(config)

            self.assertEqual(ret, (True, "Valid beacon configuration"))

            ret = service_beacon.beacon(config)
            self.assertEqual(
                ret,
                [
                    {
                        "service_name": "salt-master",
                        "tag": "salt-master",
                        "salt-master": {"running": False},
                    }
                ],
            )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.kmod as kmod

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class KmodTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.kmod
    """

    def setup_loader_modules(self):
        return {kmod: {}}

    # 'present' function tests: 2

    def test_present(self):
        """
        Test to ensure that the specified kernel module is loaded.
        """
        name = "cheese"
        ret = {"name": name, "result": True, "comment": "", "changes": {}}

        mock_mod_list = MagicMock(return_value=[name])
        with patch.dict(kmod.__salt__, {"kmod.mod_list": mock_mod_list}):
            comment = "Kernel module {0} is already present".format(name)
            ret.update({"comment": comment})
            self.assertDictEqual(kmod.present(name), ret)

        mock_mod_list = MagicMock(return_value=[])
        with patch.dict(kmod.__salt__, {"kmod.mod_list": mock_mod_list}):
            with patch.dict(kmod.__opts__, {"test": True}):
                comment = "Kernel module {0} is set to be loaded".format(name)
                ret.update({"comment": comment, "result": None})
                self.assertDictEqual(kmod.present(name), ret)

        mock_mod_list = MagicMock(return_value=[])
        mock_available = MagicMock(return_value=[name])
        mock_load = MagicMock(return_value=[name])
        with patch.dict(
            kmod.__salt__,
            {
                "kmod.mod_list": mock_mod_list,
                "kmod.available": mock_available,
                "kmod.load": mock_load,
            },
        ):
            with patch.dict(kmod.__opts__, {"test": False}):
                comment = "Loaded kernel module {0}".format(name)
                ret.update(
                    {"comment": comment, "result": True, "changes": {name: "loaded"}}
                )
                self.assertDictEqual(kmod.present(name), ret)

    def test_present_multi(self):
        """
        Test to ensure that multiple kernel modules are loaded.
        """
        name = "salted kernel"
        mods = ["cheese", "crackers"]
        ret = {"name": name, "result": True, "changes": {}}

        mock_mod_list = MagicMock(return_value=mods)
        with patch.dict(kmod.__salt__, {"kmod.mod_list": mock_mod_list}):
            call_ret = kmod.present(name, mods=mods)

            # Check comment independently: makes test more stable on PY3
            comment = call_ret.pop("comment")
            self.assertIn("cheese", comment)
            self.assertIn("crackers", comment)
            self.assertIn("are already present", comment)

            # Assert against all other dictionary key/values
            self.assertDictEqual(ret, call_ret)

        mock_mod_list = MagicMock(return_value=[])
        with patch.dict(kmod.__salt__, {"kmod.mod_list": mock_mod_list}):
            with patch.dict(kmod.__opts__, {"test": True}):
                call_ret = kmod.present(name, mods=mods)
                ret.update({"result": None})

                # Check comment independently: makes test more stable on PY3
                comment = call_ret.pop("comment")
                self.assertIn("cheese", comment)
                self.assertIn("crackers", comment)
                self.assertIn("are set to be loaded", comment)

                # Assert against all other dictionary key/values
                self.assertDictEqual(ret, call_ret)

        mock_mod_list = MagicMock(return_value=[])
        mock_available = MagicMock(return_value=mods)
        mock_load = MagicMock(return_value=mods)
        with patch.dict(
            kmod.__salt__,
            {
                "kmod.mod_list": mock_mod_list,
                "kmod.available": mock_available,
                "kmod.load": mock_load,
            },
        ):
            with patch.dict(kmod.__opts__, {"test": False}):
                call_ret = kmod.present(name, mods=mods)
                ret.update(
                    {"result": True, "changes": {mods[0]: "loaded", mods[1]: "loaded"}}
                )

                # Check comment independently: makes test more stable on PY3
                comment = call_ret.pop("comment")
                self.assertIn("cheese", comment)
                self.assertIn("crackers", comment)
                self.assertIn("Loaded kernel modules", comment)

                # Assert against all other dictionary key/values
                self.assertDictEqual(ret, call_ret)

    # 'absent' function tests: 2

    def test_absent(self):
        """
        Test to verify that the named kernel module is not loaded.
        """
        name = "cheese"
        ret = {"name": name, "result": True, "comment": "", "changes": {}}

        mock_mod_list = MagicMock(return_value=[name])
        with patch.dict(kmod.__salt__, {"kmod.mod_list": mock_mod_list}):
            with patch.dict(kmod.__opts__, {"test": True}):
                comment = "Kernel module {0} is set to be removed".format(name)
                ret.update({"comment": comment, "result": None})
                self.assertDictEqual(kmod.absent(name), ret)

        mock_mod_list = MagicMock(return_value=[name])
        mock_remove = MagicMock(return_value=[name])
        with patch.dict(
            kmod.__salt__, {"kmod.mod_list": mock_mod_list, "kmod.remove": mock_remove}
        ):
            with patch.dict(kmod.__opts__, {"test": False}):
                comment = "Removed kernel module {0}".format(name)
                ret.update(
                    {"comment": comment, "result": True, "changes": {name: "removed"}}
                )
                self.assertDictEqual(kmod.absent(name), ret)

        mock_mod_list = MagicMock(return_value=[])
        with patch.dict(kmod.__salt__, {"kmod.mod_list": mock_mod_list}):
            with patch.dict(kmod.__opts__, {"test": True}):
                comment = "Kernel module {0} is already removed".format(name)
                ret.update({"comment": comment, "result": True, "changes": {}})
                self.assertDictEqual(kmod.absent(name), ret)

    def test_absent_multi(self):
        """
        Test to verify that multiple kernel modules are not loaded.
        """
        name = "salted kernel"
        mods = ["cheese", "crackers"]
        ret = {"name": name, "result": True, "changes": {}}

        mock_mod_list = MagicMock(return_value=mods)
        with patch.dict(kmod.__salt__, {"kmod.mod_list": mock_mod_list}):
            with patch.dict(kmod.__opts__, {"test": True}):
                ret.update({"result": None})
                call_ret = kmod.absent(name, mods=mods)

                # Check comment independently: makes test more stable on PY3
                comment = call_ret.pop("comment")
                self.assertIn("cheese", comment)
                self.assertIn("crackers", comment)
                self.assertIn("are set to be removed", comment)

                # Assert against all other dictionary key/values
                self.assertDictEqual(ret, call_ret)

        mock_mod_list = MagicMock(return_value=mods)
        mock_remove = MagicMock(return_value=mods)
        with patch.dict(
            kmod.__salt__, {"kmod.mod_list": mock_mod_list, "kmod.remove": mock_remove}
        ):
            with patch.dict(kmod.__opts__, {"test": False}):
                call_ret = kmod.absent(name, mods=mods)
                ret.update(
                    {
                        "result": True,
                        "changes": {mods[0]: "removed", mods[1]: "removed"},
                    }
                )

                # Check comment independently: makes test more stable on PY3
                comment = call_ret.pop("comment")
                self.assertIn("cheese", comment)
                self.assertIn("crackers", comment)
                self.assertIn("Removed kernel modules", comment)

                # Assert against all other dictionary key/values
                self.assertDictEqual(ret, call_ret)

        mock_mod_list = MagicMock(return_value=[])
        with patch.dict(kmod.__salt__, {"kmod.mod_list": mock_mod_list}):
            with patch.dict(kmod.__opts__, {"test": True}):
                comment = "Kernel modules {0} are already removed".format(
                    ", ".join(mods)
                )
                ret.update({"comment": comment, "result": True, "changes": {}})
                self.assertDictEqual(kmod.absent(name, mods=mods), ret)



# -*- coding: utf-8 -*-
"""
Integration tests for the vault modules
"""
from __future__ import absolute_import, print_function, unicode_literals

import inspect
import logging
import time

import salt.utils.path
from tests.support.case import ModuleCase, ShellCase
from tests.support.helpers import destructiveTest, flaky, slowTest
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import skipIf

log = logging.getLogger(__name__)


@skipIf(not salt.utils.path.which("dockerd"), "Docker not installed")
@skipIf(not salt.utils.path.which("vault"), "Vault not installed")
class VaultTestCase(ModuleCase, ShellCase):
    """
    Test vault module
    """

    count = 0

    def setUp(self):
        """
        SetUp vault container
        """

        vault_binary = salt.utils.path.which("vault")
        if VaultTestCase.count == 0:
            config = '{"backend": {"file": {"path": "/vault/file"}}, "default_lease_ttl": "168h", "max_lease_ttl": "720h"}'
            self.run_state("docker_image.present", name="vault", tag="0.9.6")
            self.run_state(
                "docker_container.running",
                name="vault",
                image="vault:0.9.6",
                port_bindings="8200:8200",
                environment={
                    "VAULT_DEV_ROOT_TOKEN_ID": "testsecret",
                    "VAULT_LOCAL_CONFIG": config,
                },
                cap_add="IPC_LOCK",
            )
            time.sleep(5)
            ret = self.run_function(
                "cmd.retcode",
                cmd="{} login token=testsecret".format(vault_binary),
                env={"VAULT_ADDR": "http://127.0.0.1:8200"},
            )
            login_attempts = 1
            # If the login failed, container might have stopped
            # attempt again, maximum of three times before
            # skipping.
            while ret != 0:
                self.run_state(
                    "docker_container.running",
                    name="vault",
                    image="vault:0.9.6",
                    port_bindings="8200:8200",
                    environment={
                        "VAULT_DEV_ROOT_TOKEN_ID": "testsecret",
                        "VAULT_LOCAL_CONFIG": config,
                    },
                    cap_add="IPC_LOCK",
                )
                time.sleep(5)
                ret = self.run_function(
                    "cmd.retcode",
                    cmd="{} login token=testsecret".format(vault_binary),
                    env={"VAULT_ADDR": "http://127.0.0.1:8200"},
                )
                login_attempts += 1
                if login_attempts >= 3:
                    self.skipTest("unable to login to vault")
            ret = self.run_function(
                "cmd.retcode",
                cmd="{} policy write testpolicy {}/vault.hcl".format(
                    vault_binary, RUNTIME_VARS.FILES
                ),
                env={"VAULT_ADDR": "http://127.0.0.1:8200"},
            )
            if ret != 0:
                self.skipTest("unable to assign policy to vault")
        VaultTestCase.count += 1

    def tearDown(self):
        """
        TearDown vault container
        """

        def count_tests(funcobj):
            return (
                inspect.ismethod(funcobj)
                or inspect.isfunction(funcobj)
                and funcobj.__name__.startswith("test_")
            )

        numtests = len(inspect.getmembers(VaultTestCase, predicate=count_tests))
        if VaultTestCase.count >= numtests:
            self.run_state("docker_container.stopped", name="vault")
            self.run_state("docker_container.absent", name="vault")
            self.run_state("docker_image.absent", name="vault", force=True)

    @flaky
    @slowTest
    def test_sdb(self):
        set_output = self.run_function(
            "sdb.set", uri="sdb://sdbvault/secret/test/test_sdb/foo", value="bar"
        )
        self.assertEqual(set_output, True)
        get_output = self.run_function(
            "sdb.get", arg=["sdb://sdbvault/secret/test/test_sdb/foo"]
        )
        self.assertEqual(get_output, "bar")

    @flaky
    @slowTest
    def test_sdb_runner(self):
        set_output = self.run_run(
            "sdb.set sdb://sdbvault/secret/test/test_sdb_runner/foo bar"
        )
        self.assertEqual(set_output, ["True"])
        get_output = self.run_run(
            "sdb.get sdb://sdbvault/secret/test/test_sdb_runner/foo"
        )
        self.assertEqual(get_output, ["bar"])

    @flaky
    @slowTest
    def test_config(self):
        set_output = self.run_function(
            "sdb.set", uri="sdb://sdbvault/secret/test/test_pillar_sdb/foo", value="bar"
        )
        self.assertEqual(set_output, True)
        get_output = self.run_function("config.get", arg=["test_vault_pillar_sdb"])
        self.assertEqual(get_output, "bar")


@destructiveTest
@skipIf(not salt.utils.path.which("dockerd"), "Docker not installed")
@skipIf(not salt.utils.path.which("vault"), "Vault not installed")
class VaultTestCaseCurrent(ModuleCase, ShellCase):
    """
    Test vault module
    """

    count = 0

    def setUp(self):
        """
        SetUp vault container
        """
        if self.count == 0:
            config = '{"backend": {"file": {"path": "/vault/file"}}, "default_lease_ttl": "168h", "max_lease_ttl": "720h"}'
            self.run_state("docker_image.present", name="vault", tag="1.3.1")
            self.run_state(
                "docker_container.running",
                name="vault",
                image="vault:1.3.1",
                port_bindings="8200:8200",
                environment={
                    "VAULT_DEV_ROOT_TOKEN_ID": "testsecret",
                    "VAULT_LOCAL_CONFIG": config,
                },
                cap_add="IPC_LOCK",
            )
            time.sleep(5)
            ret = self.run_function(
                "cmd.retcode",
                cmd="/usr/local/bin/vault login token=testsecret",
                env={"VAULT_ADDR": "http://127.0.0.1:8200"},
            )
            login_attempts = 1
            # If the login failed, container might have stopped
            # attempt again, maximum of three times before
            # skipping.
            while ret != 0:
                self.run_state(
                    "docker_container.running",
                    name="vault",
                    image="vault:1.3.1",
                    port_bindings="8200:8200",
                    environment={
                        "VAULT_DEV_ROOT_TOKEN_ID": "testsecret",
                        "VAULT_LOCAL_CONFIG": config,
                    },
                    cap_add="IPC_LOCK",
                )
                time.sleep(5)
                ret = self.run_function(
                    "cmd.retcode",
                    cmd="/usr/local/bin/vault login token=testsecret",
                    env={"VAULT_ADDR": "http://127.0.0.1:8200"},
                )
                login_attempts += 1
                if login_attempts >= 3:
                    self.skipTest("unable to login to vault")
            ret = self.run_function(
                "cmd.retcode",
                cmd="/usr/local/bin/vault policy write testpolicy {0}/vault.hcl".format(
                    RUNTIME_VARS.FILES
                ),
                env={"VAULT_ADDR": "http://127.0.0.1:8200"},
            )
            if ret != 0:
                self.skipTest("unable to assign policy to vault")
        self.count += 1

    def tearDown(self):
        """
        TearDown vault container
        """

        def count_tests(funcobj):
            return (
                inspect.ismethod(funcobj)
                or inspect.isfunction(funcobj)
                and funcobj.__name__.startswith("test_")
            )

        numtests = len(inspect.getmembers(VaultTestCaseCurrent, predicate=count_tests))
        if self.count >= numtests:
            self.run_state("docker_container.stopped", name="vault")
            self.run_state("docker_container.absent", name="vault")
            self.run_state("docker_image.absent", name="vault", force=True)

    @flaky
    @slowTest
    def test_sdb_kv2(self):
        set_output = self.run_function(
            "sdb.set", uri="sdb://sdbvault/secret/test/test_sdb/foo", value="bar"
        )
        self.assertEqual(set_output, True)
        get_output = self.run_function(
            "sdb.get", arg=["sdb://sdbvault/secret/test/test_sdb/foo"]
        )
        self.assertEqual(get_output, "bar")

    @flaky
    @slowTest
    def test_sdb_runner_kv2(self):
        set_output = self.run_run(
            "sdb.set sdb://sdbvault/secret/test/test_sdb_runner/foo bar"
        )
        self.assertEqual(set_output, ["True"])
        get_output = self.run_run(
            "sdb.get sdb://sdbvault/secret/test/test_sdb_runner/foo"
        )
        self.assertEqual(get_output, ["bar"])

    @flaky
    @slowTest
    def test_config_kv2(self):
        set_output = self.run_function(
            "sdb.set", uri="sdb://sdbvault/secret/test/test_pillar_sdb/foo", value="bar"
        )
        self.assertEqual(set_output, True)
        get_output = self.run_function("config.get", arg=["test_vault_pillar_sdb"])
        self.assertEqual(get_output, "bar")



# coding: utf-8

# Python libs
from __future__ import absolute_import

# Salt libs
import salt.beacons.sensehat as sensehat
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock

# Salt testing libs
from tests.support.unit import TestCase


class SensehatBeaconTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test case for salt.beacons.[s]
    """

    def setup_loader_modules(self):

        self.HUMIDITY_MOCK = MagicMock(return_value=80)
        self.TEMPERATURE_MOCK = MagicMock(return_value=30)
        self.PRESSURE_MOCK = MagicMock(return_value=1500)

        self.addCleanup(delattr, self, "HUMIDITY_MOCK")
        self.addCleanup(delattr, self, "TEMPERATURE_MOCK")
        self.addCleanup(delattr, self, "PRESSURE_MOCK")

        return {
            sensehat: {
                "__context__": {},
                "__salt__": {
                    "sensehat.get_humidity": self.HUMIDITY_MOCK,
                    "sensehat.get_temperature": self.TEMPERATURE_MOCK,
                    "sensehat.get_pressure": self.PRESSURE_MOCK,
                },
            }
        }

    def test_non_list_config(self):
        config = {}

        ret = sensehat.validate(config)

        self.assertEqual(
            ret, (False, "Configuration for sensehat beacon must be a list.")
        )

    def test_empty_config(self):
        config = [{}]

        ret = sensehat.validate(config)

        self.assertEqual(
            ret, (False, "Configuration for sensehat beacon requires sensors.")
        )

    def test_sensehat_humidity_match(self):

        config = [{"sensors": {"humidity": "70%"}}]

        ret = sensehat.validate(config)
        self.assertEqual(ret, (True, "Valid beacon configuration"))

        ret = sensehat.beacon(config)
        self.assertEqual(ret, [{"tag": "sensehat/humidity", "humidity": 80}])

    def test_sensehat_temperature_match(self):

        config = [{"sensors": {"temperature": 20}}]

        ret = sensehat.validate(config)
        self.assertEqual(ret, (True, "Valid beacon configuration"))

        ret = sensehat.beacon(config)
        self.assertEqual(ret, [{"tag": "sensehat/temperature", "temperature": 30}])

    def test_sensehat_temperature_match_range(self):

        config = [{"sensors": {"temperature": [20, 29]}}]

        ret = sensehat.validate(config)
        self.assertEqual(ret, (True, "Valid beacon configuration"))

        ret = sensehat.beacon(config)
        self.assertEqual(ret, [{"tag": "sensehat/temperature", "temperature": 30}])

    def test_sensehat_pressure_match(self):

        config = [{"sensors": {"pressure": "1400"}}]

        ret = sensehat.validate(config)
        self.assertEqual(ret, (True, "Valid beacon configuration"))

        ret = sensehat.beacon(config)
        self.assertEqual(ret, [{"tag": "sensehat/pressure", "pressure": 1500}])

    def test_sensehat_no_match(self):

        config = [{"sensors": {"pressure": "1600"}}]

        ret = sensehat.validate(config)
        self.assertEqual(ret, (True, "Valid beacon configuration"))

        ret = sensehat.beacon(config)
        self.assertEqual(ret, [])



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.win_certutil as certutil

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class CertUtilTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {certutil: {}}

    def test_add_serial(self):
        """
            Test adding a certificate to specified certificate store
        """
        expected = {
            "changes": {"added": "/path/to/cert.cer"},
            "comment": "",
            "name": "/path/to/cert.cer",
            "result": True,
        }

        cache_mock = MagicMock(return_value="/tmp/cert.cer")
        get_cert_serial_mock = MagicMock(return_value="ABCDEF")
        get_store_serials_mock = MagicMock(return_value=["123456"])
        add_mock = MagicMock(return_value="Added successfully")
        with patch.dict(
            certutil.__salt__,
            {
                "cp.cache_file": cache_mock,
                "certutil.get_cert_serial": get_cert_serial_mock,
                "certutil.get_stored_cert_serials": get_store_serials_mock,
                "certutil.add_store": add_mock,
            },
        ):
            out = certutil.add_store("/path/to/cert.cer", "TrustedPublisher")
            cache_mock.assert_called_once_with("/path/to/cert.cer", "base")
            get_cert_serial_mock.assert_called_once_with("/tmp/cert.cer")
            get_store_serials_mock.assert_called_once_with("TrustedPublisher")
            add_mock.assert_called_once_with("/path/to/cert.cer", "TrustedPublisher")
            self.assertEqual(expected, out)

    def test_add_serial_missing(self):
        """
            Test adding a certificate to specified certificate store when the file doesn't exist
        """
        expected = {
            "changes": {},
            "comment": "Certificate file not found.",
            "name": "/path/to/cert.cer",
            "result": False,
        }

        cache_mock = MagicMock(return_value=False)
        get_cert_serial_mock = MagicMock(return_value="ABCDEF")
        get_store_serials_mock = MagicMock(return_value=["123456"])
        add_mock = MagicMock(return_value="Added successfully")
        with patch.dict(
            certutil.__salt__,
            {
                "cp.cache_file": cache_mock,
                "certutil.get_cert_serial": get_cert_serial_mock,
                "certutil.get_stored_cert_serials": get_store_serials_mock,
                "certutil.add_store": add_mock,
            },
        ):
            out = certutil.add_store("/path/to/cert.cer", "TrustedPublisher")
            cache_mock.assert_called_once_with("/path/to/cert.cer", "base")
            assert not get_cert_serial_mock.called
            assert not get_store_serials_mock.called
            assert not add_mock.called
            self.assertEqual(expected, out)

    def test_add_serial_exists(self):
        """
            Test adding a certificate to specified certificate store when the cert already exists
        """
        expected = {
            "changes": {},
            "comment": "/path/to/cert.cer already stored.",
            "name": "/path/to/cert.cer",
            "result": True,
        }

        cache_mock = MagicMock(return_value="/tmp/cert.cer")
        get_cert_serial_mock = MagicMock(return_value="ABCDEF")
        get_store_serials_mock = MagicMock(return_value=["123456", "ABCDEF"])
        add_mock = MagicMock(return_value="Added successfully")
        with patch.dict(
            certutil.__salt__,
            {
                "cp.cache_file": cache_mock,
                "certutil.get_cert_serial": get_cert_serial_mock,
                "certutil.get_stored_cert_serials": get_store_serials_mock,
                "certutil.add_store": add_mock,
            },
        ):
            out = certutil.add_store("/path/to/cert.cer", "TrustedPublisher")
            cache_mock.assert_called_once_with("/path/to/cert.cer", "base")
            get_cert_serial_mock.assert_called_once_with("/tmp/cert.cer")
            get_store_serials_mock.assert_called_once_with("TrustedPublisher")
            assert not add_mock.called
            self.assertEqual(expected, out)

    def test_add_serial_fail(self):
        """
            Test adding a certificate when the add fails
        """
        expected = {
            "changes": {},
            "comment": "Failed to store certificate /path/to/cert.cer",
            "name": "/path/to/cert.cer",
            "result": False,
        }

        cache_mock = MagicMock(return_value="/tmp/cert.cer")
        get_cert_serial_mock = MagicMock(return_value="ABCDEF")
        get_store_serials_mock = MagicMock(return_value=["123456"])
        add_mock = MagicMock(return_value="Failed")
        with patch.dict(
            certutil.__salt__,
            {
                "cp.cache_file": cache_mock,
                "certutil.get_cert_serial": get_cert_serial_mock,
                "certutil.get_stored_cert_serials": get_store_serials_mock,
                "certutil.add_store": add_mock,
            },
        ):
            out = certutil.add_store("/path/to/cert.cer", "TrustedPublisher")
            cache_mock.assert_called_once_with("/path/to/cert.cer", "base")
            get_cert_serial_mock.assert_called_once_with("/tmp/cert.cer")
            get_store_serials_mock.assert_called_once_with("TrustedPublisher")
            add_mock.assert_called_once_with("/path/to/cert.cer", "TrustedPublisher")
            self.assertEqual(expected, out)

    def test_del_serial(self):
        """
            Test deleting a certificate from a specified certificate store
        """
        expected = {
            "changes": {"removed": "/path/to/cert.cer"},
            "comment": "",
            "name": "/path/to/cert.cer",
            "result": True,
        }

        cache_mock = MagicMock(return_value="/tmp/cert.cer")
        get_cert_serial_mock = MagicMock(return_value="ABCDEF")
        get_store_serials_mock = MagicMock(return_value=["123456", "ABCDEF"])
        del_mock = MagicMock(return_value="Removed successfully")
        with patch.dict(
            certutil.__salt__,
            {
                "cp.cache_file": cache_mock,
                "certutil.get_cert_serial": get_cert_serial_mock,
                "certutil.get_stored_cert_serials": get_store_serials_mock,
                "certutil.del_store": del_mock,
            },
        ):
            out = certutil.del_store("/path/to/cert.cer", "TrustedPublisher")
            cache_mock.assert_called_once_with("/path/to/cert.cer", "base")
            get_cert_serial_mock.assert_called_once_with("/tmp/cert.cer")
            get_store_serials_mock.assert_called_once_with("TrustedPublisher")
            del_mock.assert_called_once_with("/tmp/cert.cer", "TrustedPublisher")
            self.assertEqual(expected, out)

    def test_del_serial_missing(self):
        """
            Test deleting a certificate to specified certificate store when the file doesn't exist
        """
        expected = {
            "changes": {},
            "comment": "Certificate file not found.",
            "name": "/path/to/cert.cer",
            "result": False,
        }

        cache_mock = MagicMock(return_value=False)
        get_cert_serial_mock = MagicMock(return_value="ABCDEF")
        get_store_serials_mock = MagicMock(return_value=["123456"])
        del_mock = MagicMock(return_value="Added successfully")
        with patch.dict(
            certutil.__salt__,
            {
                "cp.cache_file": cache_mock,
                "certutil.get_cert_serial": get_cert_serial_mock,
                "certutil.get_stored_cert_serials": get_store_serials_mock,
                "certutil.del_store": del_mock,
            },
        ):
            out = certutil.del_store("/path/to/cert.cer", "TrustedPublisher")
            cache_mock.assert_called_once_with("/path/to/cert.cer", "base")
            assert not get_cert_serial_mock.called
            assert not get_store_serials_mock.called
            assert not del_mock.called
            self.assertEqual(expected, out)

    def test_del_serial_doesnt_exists(self):
        """
            Test deleting a certificate to specified certificate store when the cert doesn't exists
        """
        expected = {
            "changes": {},
            "comment": "/path/to/cert.cer already removed.",
            "name": "/path/to/cert.cer",
            "result": True,
        }

        cache_mock = MagicMock(return_value="/tmp/cert.cer")
        get_cert_serial_mock = MagicMock(return_value="ABCDEF")
        get_store_serials_mock = MagicMock(return_value=["123456"])
        del_mock = MagicMock(return_value="Added successfully")
        with patch.dict(
            certutil.__salt__,
            {
                "cp.cache_file": cache_mock,
                "certutil.get_cert_serial": get_cert_serial_mock,
                "certutil.get_stored_cert_serials": get_store_serials_mock,
                "certutil.del_store": del_mock,
            },
        ):
            out = certutil.del_store("/path/to/cert.cer", "TrustedPublisher")
            cache_mock.assert_called_once_with("/path/to/cert.cer", "base")
            get_cert_serial_mock.assert_called_once_with("/tmp/cert.cer")
            get_store_serials_mock.assert_called_once_with("TrustedPublisher")
            assert not del_mock.called
            self.assertEqual(expected, out)

    def test_del_serial_fail(self):
        """
            Test deleting a certificate from the store when the delete fails
        """
        expected = {
            "changes": {},
            "comment": "Failed to remove the certificate /path/to/cert.cer",
            "name": "/path/to/cert.cer",
            "result": False,
        }

        cache_mock = MagicMock(return_value="/tmp/cert.cer")
        get_cert_serial_mock = MagicMock(return_value="ABCDEF")
        get_store_serials_mock = MagicMock(return_value=["123456", "ABCDEF"])
        del_mock = MagicMock(return_value="Failed")
        with patch.dict(
            certutil.__salt__,
            {
                "cp.cache_file": cache_mock,
                "certutil.get_cert_serial": get_cert_serial_mock,
                "certutil.get_stored_cert_serials": get_store_serials_mock,
                "certutil.del_store": del_mock,
            },
        ):
            out = certutil.del_store("/path/to/cert.cer", "TrustedPublisher")
            cache_mock.assert_called_once_with("/path/to/cert.cer", "base")
            get_cert_serial_mock.assert_called_once_with("/tmp/cert.cer")
            get_store_serials_mock.assert_called_once_with("TrustedPublisher")
            del_mock.assert_called_once_with("/tmp/cert.cer", "TrustedPublisher")
            self.assertEqual(expected, out)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)


    tests.integration.shell.cp
    ~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

from __future__ import absolute_import

import logging
import os
import pipes

import pytest
import salt.utils.files
import salt.utils.platform
import salt.utils.yaml
from salt.ext import six
from tests.support.case import ShellCase
from tests.support.helpers import slowTest
from tests.support.mixins import ShellCaseCommonTestsMixin
from tests.support.runtests import RUNTIME_VARS

log = logging.getLogger(__name__)


@pytest.mark.windows_whitelisted
class CopyTest(ShellCase, ShellCaseCommonTestsMixin):

    _call_binary_ = "salt-cp"

    @slowTest
    def test_cp_testfile(self):
        """
        test salt-cp
        """
        minions = []
        for line in self.run_salt('--out yaml "*" test.ping'):
            if not line:
                continue
            data = salt.utils.yaml.safe_load(line)
            minions.extend(data.keys())

        self.assertNotEqual(minions, [])

        testfile = os.path.abspath(
            os.path.join(
                os.path.dirname(os.path.dirname(__file__)),
                "files",
                "file",
                "base",
                "testfile",
            )
        )
        with salt.utils.files.fopen(testfile, "r") as fh_:
            testfile_contents = fh_.read()

        def quote(arg):
            if salt.utils.platform.is_windows():
                return arg
            return pipes.quote(arg)

        for idx, minion in enumerate(minions):
            if "localhost" in minion:
                continue
            ret = self.run_salt(
                "--out yaml {0} file.directory_exists {1}".format(
                    quote(minion), RUNTIME_VARS.TMP
                )
            )
            data = salt.utils.yaml.safe_load("\n".join(ret))
            if data[minion] is False:
                ret = self.run_salt(
                    "--out yaml {0} file.makedirs {1}".format(
                        quote(minion), RUNTIME_VARS.TMP
                    )
                )

                data = salt.utils.yaml.safe_load("\n".join(ret))
                self.assertTrue(data[minion])

            minion_testfile = os.path.join(
                RUNTIME_VARS.TMP, "cp_{0}_testfile".format(idx)
            )

            ret = self.run_cp(
                "--out pprint {0} {1} {2}".format(
                    quote(minion), quote(testfile), quote(minion_testfile),
                )
            )

            data = eval("\n".join(ret), {}, {})  # pylint: disable=eval-used
            for part in six.itervalues(data):
                key = minion_testfile
                self.assertTrue(part[key])

            ret = self.run_salt(
                "--out yaml {0} file.file_exists {1}".format(
                    quote(minion), quote(minion_testfile)
                )
            )
            data = salt.utils.yaml.safe_load("\n".join(ret))
            self.assertTrue(data[minion])

            ret = self.run_salt(
                "--out yaml {0} file.contains {1} {2}".format(
                    quote(minion), quote(minion_testfile), quote(testfile_contents)
                )
            )
            data = salt.utils.yaml.safe_load("\n".join(ret))
            self.assertTrue(data[minion])
            ret = self.run_salt(
                "--out yaml {0} file.remove {1}".format(
                    quote(minion), quote(minion_testfile)
                )
            )
            data = salt.utils.yaml.safe_load("\n".join(ret))
            self.assertTrue(data[minion])



# coding: utf-8

# Python libs
from __future__ import absolute_import

# Salt libs
import salt.beacons.glxinfo as glxinfo

# Import test suite libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import Mock, patch

# Salt testing libs
from tests.support.unit import TestCase


class GLXInfoBeaconTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test case for salt.beacons.glxinfo
    """

    def setup_loader_modules(self):
        return {glxinfo: {"last_state": {}}}

    def test_no_glxinfo_command(self):
        with patch("salt.utils.path.which") as mock:
            mock.return_value = None

            ret = glxinfo.__virtual__()

            mock.assert_called_once_with("glxinfo")
            self.assertFalse(ret)

    def test_with_glxinfo_command(self):
        with patch("salt.utils.path.which") as mock:
            mock.return_value = "/usr/bin/glxinfo"

            ret = glxinfo.__virtual__()

            mock.assert_called_once_with("glxinfo")
            self.assertEqual(ret, "glxinfo")

    def test_non_list_config(self):
        config = {}

        ret = glxinfo.validate(config)

        self.assertEqual(
            ret, (False, "Configuration for glxinfo beacon must be a list.")
        )

    def test_no_user(self):
        config = [{"screen_event": True}]

        _expected = (
            False,
            "Configuration for glxinfo beacon must "
            "include a user as glxinfo is not "
            "available to root.",
        )
        ret = glxinfo.validate(config)
        self.assertEqual(ret, _expected)

    def test_screen_state(self):
        config = [{"screen_event": True, "user": "frank"}]

        mock = Mock(return_value=0)
        with patch.dict(glxinfo.__salt__, {"cmd.retcode": mock}):

            ret = glxinfo.validate(config)
            self.assertEqual(ret, (True, "Valid beacon configuration"))

            ret = glxinfo.beacon(config)
            self.assertEqual(ret, [{"tag": "screen_event", "screen_available": True}])
            mock.assert_called_once_with(
                "DISPLAY=:0 glxinfo", runas="frank", python_shell=True
            )

    def test_screen_state_missing(self):
        config = [{"screen_event": True, "user": "frank"}]

        mock = Mock(return_value=255)
        with patch.dict(glxinfo.__salt__, {"cmd.retcode": mock}):
            ret = glxinfo.validate(config)
            self.assertEqual(ret, (True, "Valid beacon configuration"))

            ret = glxinfo.beacon(config)
            self.assertEqual(ret, [{"tag": "screen_event", "screen_available": False}])

    def test_screen_state_no_repeat(self):
        config = [{"screen_event": True, "user": "frank"}]

        mock = Mock(return_value=255)
        with patch.dict(glxinfo.__salt__, {"cmd.retcode": mock}):
            ret = glxinfo.validate(config)
            self.assertEqual(ret, (True, "Valid beacon configuration"))

            ret = glxinfo.beacon(config)
            self.assertEqual(ret, [{"tag": "screen_event", "screen_available": False}])

            ret = glxinfo.beacon(config)
            self.assertEqual(ret, [])

    def test_screen_state_change(self):
        config = [{"screen_event": True, "user": "frank"}]

        mock = Mock(side_effect=[255, 0])
        with patch.dict(glxinfo.__salt__, {"cmd.retcode": mock}):
            ret = glxinfo.validate(config)
            self.assertEqual(ret, (True, "Valid beacon configuration"))

            ret = glxinfo.beacon(config)
            self.assertEqual(ret, [{"tag": "screen_event", "screen_available": False}])

            ret = glxinfo.beacon(config)
            self.assertEqual(ret, [{"tag": "screen_event", "screen_available": True}])



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.smtp as smtp

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class SMTPRecipientsRefused(Exception):
    """
    Mock SMTPRecipientsRefused class
    """

    def __init__(self, msg):
        super(SMTPRecipientsRefused, self).__init__(msg)
        self.smtp_error = msg


class SMTPHeloError(Exception):
    """
    Mock SMTPHeloError class
    """

    def __init__(self, msg):
        super(SMTPHeloError, self).__init__(msg)
        self.smtp_error = msg


class SMTPSenderRefused(Exception):
    """
    Mock SMTPSenderRefused class
    """

    def __init__(self, msg):
        super(SMTPSenderRefused, self).__init__(msg)
        self.smtp_error = msg


class SMTPDataError(Exception):
    """
    Mock SMTPDataError class
    """

    def __init__(self, msg):
        super(SMTPDataError, self).__init__(msg)
        self.smtp_error = msg


class SMTPException(Exception):
    """
    Mock SMTPException class
    """

    def __init__(self, msg):
        super(SMTPException, self).__init__(msg)
        self.smtp_error = msg


class SMTPAuthenticationError(Exception):
    """
    Mock SMTPAuthenticationError class
    """

    def __init__(self, msg):
        super(SMTPAuthenticationError, self).__init__(msg)
        self.smtp_error = msg


class MockSMTPSSL(object):
    """
    Mock SMTP_SSL class
    """

    flag = None

    def __init__(self, server):
        pass

    def sendmail(self, sender, recipient, msg):
        """
        Mock sendmail method
        """
        if self.flag == 1:
            raise SMTPRecipientsRefused("All recipients were refused.")
        elif self.flag == 2:
            raise SMTPHeloError("Helo error")
        elif self.flag == 3:
            raise SMTPSenderRefused("Sender Refused")
        elif self.flag == 4:
            raise SMTPDataError("Data error")
        return (sender, recipient, msg)

    def login(self, username, password):
        """
        Mock login method
        """
        if self.flag == 5:
            raise SMTPAuthenticationError("SMTP Authentication Failure")
        return (username, password)

    @staticmethod
    def quit():
        """
        Mock quit method
        """
        return True


class MockSMTP(object):
    """
    Mock SMTP class
    """

    flag = None

    def __init__(self, server):
        pass

    @staticmethod
    def ehlo():
        """
        Mock ehlo method
        """
        return True

    @staticmethod
    def has_extn(name):
        """
        Mock has_extn method
        """
        return name

    def starttls(self):
        """
        Mock starttls method
        """
        if self.flag == 1:
            raise SMTPHeloError("Helo error")
        elif self.flag == 2:
            raise SMTPException("Exception error")
        elif self.flag == 3:
            raise RuntimeError
        return True

    def sendmail(self, sender, recipient, msg):
        """
        Mock sendmail method
        """
        if self.flag == 1:
            raise SMTPRecipientsRefused("All recipients were refused.")
        elif self.flag == 2:
            raise SMTPHeloError("Helo error")
        elif self.flag == 3:
            raise SMTPSenderRefused("Sender Refused")
        elif self.flag == 4:
            raise SMTPDataError("Data error")
        return (sender, recipient, msg)

    @staticmethod
    def quit():
        """
        Mock quit method
        """
        return True


class MockGaierror(Exception):
    """
    Mock MockGaierror class
    """

    def __init__(self, msg):
        super(MockGaierror, self).__init__(msg)
        self.smtp_error = msg


class MockSocket(object):
    """
    Mock Socket class
    """

    def __init__(self):
        self.gaierror = MockGaierror


class MockSmtplib(object):
    """
    Mock smtplib class
    """

    flag = None

    def __init__(self):
        self.SMTPRecipientsRefused = SMTPRecipientsRefused
        self.SMTPHeloError = SMTPHeloError
        self.SMTPSenderRefused = SMTPSenderRefused
        self.SMTPDataError = SMTPDataError
        self.SMTPException = SMTPException
        self.SMTPAuthenticationError = SMTPAuthenticationError
        self.server = None

    def SMTP_SSL(self, server):
        """
        Mock SMTP_SSL method
        """
        self.server = server
        if self.flag == 1:
            raise MockGaierror("gaierror")
        return MockSMTPSSL("server")

    def SMTP(self, server):
        """
        Mock SMTP method
        """
        self.server = server
        if self.flag == 1:
            raise MockGaierror("gaierror")
        return MockSMTP("server")


class SmtpTestCase(TestCase, LoaderModuleMockMixin):
    """
    TestCase for salt.modules.smtp
    """

    def setup_loader_modules(self):
        return {smtp: {"socket": MockSocket(), "smtplib": MockSmtplib()}}

    # 'send_msg' function tests: 1

    def test_send_msg(self):
        """
        Tests if it send a message to an SMTP recipient.
        """
        mock = MagicMock(
            return_value={
                "smtp.server": "",
                "smtp.tls": "True",
                "smtp.sender": "",
                "smtp.username": "",
                "smtp.password": "",
            }
        )
        with patch.dict(smtp.__salt__, {"config.option": mock}):
            self.assertTrue(
                smtp.send_msg(
                    "admin@example.com",
                    "This is a salt module test",
                    profile="my-smtp-account",
                )
            )

            MockSMTPSSL.flag = 1
            self.assertFalse(
                smtp.send_msg(
                    "admin@example.com",
                    "This is a salt module test",
                    profile="my-smtp-account",
                )
            )

            MockSMTPSSL.flag = 2
            self.assertFalse(
                smtp.send_msg(
                    "admin@example.com",
                    "This is a salt module test",
                    profile="my-smtp-account",
                )
            )

            MockSMTPSSL.flag = 3
            self.assertFalse(
                smtp.send_msg(
                    "admin@example.com",
                    "This is a salt module test",
                    profile="my-smtp-account",
                )
            )

            MockSMTPSSL.flag = 4
            self.assertFalse(
                smtp.send_msg(
                    "admin@example.com",
                    "This is a salt module test",
                    profile="my-smtp-account",
                )
            )

        mock = MagicMock(
            return_value={
                "smtp.server": "",
                "smtp.tls": "",
                "smtp.sender": "",
                "smtp.username": "",
                "smtp.password": "",
            }
        )
        with patch.dict(smtp.__salt__, {"config.option": mock}):
            MockSMTPSSL.flag = 5
            self.assertFalse(
                smtp.send_msg(
                    "admin@example.com",
                    "This is a salt module test",
                    username="myuser",
                    password="verybadpass",
                    sender="admin@example.com",
                    server="smtp.domain.com",
                )
            )

            MockSMTP.flag = 1
            self.assertFalse(
                smtp.send_msg(
                    "admin@example.com",
                    "This is a salt module test",
                    profile="my-smtp-account",
                )
            )

            MockSMTP.flag = 2
            self.assertFalse(
                smtp.send_msg(
                    "admin@example.com",
                    "This is a salt module test",
                    profile="my-smtp-account",
                )
            )

            MockSMTP.flag = 3
            self.assertFalse(
                smtp.send_msg(
                    "admin@example.com",
                    "This is a salt module test",
                    profile="my-smtp-account",
                )
            )

            MockSmtplib.flag = 1
            self.assertFalse(
                smtp.send_msg(
                    "admin@example.com",
                    "This is a salt module test",
                    profile="my-smtp-account",
                )
            )



# -*- coding: utf-8 -*-
"""
Create an Event in VictorOps
============================

.. versionadded:: 2015.8.0

This state is useful for creating events on the
VictorOps service during state runs.

.. code-block:: yaml

    webserver-warning-message:
      victorops.create_event:
        - message_type: 'CRITICAL'
        - entity_id: 'webserver/diskspace'
        - state_message: 'Webserver diskspace is low.'
"""

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals


def __virtual__():
    """
    Only load if the victorops module is available in __salt__
    """
    if "victorops.create_event" in __salt__:
        return "victorops"
    return (False, "victorops module could not be loaded")


def create_event(name, message_type, routing_key="everyone", **kwargs):
    """
    Create an event on the VictorOps service

    .. code-block:: yaml

        webserver-warning-message:
          victorops.create_event:
            - message_type: 'CRITICAL'
            - entity_id: 'webserver/diskspace'
            - state_message: 'Webserver diskspace is low.'

        database-server-warning-message:
          victorops.create_event:
            - message_type: 'WARNING'
            - entity_id: 'db_server/load'
            - state_message: 'Database Server load is high.'
            - entity_is_host: True
            - entity_display_name: 'dbdserver.example.com'

    The following parameters are required:

    name
        This is a short description of the event.

    message_type
        One of the following values: INFO, WARNING, ACKNOWLEDGEMENT, CRITICAL, RECOVERY.

    The following parameters are optional:

        routing_key
            The key for where messages should be routed. By default, sent to 'everyone' route.

        entity_id
            The name of alerting entity. If not provided, a random name will be assigned.

        timestamp
            Timestamp of the alert in seconds since epoch. Defaults to the time the alert is received at VictorOps.

        timestamp_fmt
            The date format for the timestamp parameter.  Defaults to ''%Y-%m-%dT%H:%M:%S'.

        state_start_time
            The time this entity entered its current state (seconds since epoch). Defaults to the time alert is received.

        state_start_time_fmt
            The date format for the timestamp parameter. Defaults to '%Y-%m-%dT%H:%M:%S'.

        state_message
            Any additional status information from the alert item.

        entity_is_host
            Used within VictorOps to select the appropriate display format for the incident.

        entity_display_name
            Used within VictorOps to display a human-readable name for the entity.

        ack_message
            A user entered comment for the acknowledgment.

        ack_author
            The user that acknowledged the incident.

    """
    ret = {"name": name, "changes": {}, "result": None, "comment": ""}

    if __opts__["test"]:
        ret["comment"] = "Need to create event: {0}".format(name)
        return ret

    res = __salt__["victorops.create_event"](
        message_type=message_type, routing_key=routing_key, **kwargs
    )
    if res["result"] == "success":
        ret["result"] = True
        ret["comment"] = "Created event: {0} for entity {1}".format(
            name, res["entity_id"]
        )
    else:
        ret["result"] = False
        ret["comment"] = "Failed to create event: {0}".format(res["message"])
    return ret



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.lvm as lvm

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class LvmTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.lvm
    """

    def setup_loader_modules(self):
        return {lvm: {}}

    # 'pv_present' function tests: 1

    def test_pv_present(self):
        """
        Test to set a physical device to be used as an LVM physical volume
        """
        name = "/dev/sda5"

        comt = "Physical Volume {0} already present".format(name)

        ret = {"name": name, "changes": {}, "result": True, "comment": comt}

        mock = MagicMock(side_effect=[True, False])
        with patch.dict(lvm.__salt__, {"lvm.pvdisplay": mock}):
            self.assertDictEqual(lvm.pv_present(name), ret)

            comt = "Physical Volume {0} is set to be created".format(name)
            ret.update({"comment": comt, "result": None})
            with patch.dict(lvm.__opts__, {"test": True}):
                self.assertDictEqual(lvm.pv_present(name), ret)

    # 'pv_absent' function tests: 1

    def test_pv_absent(self):
        """
        Test to ensure that a Physical Device is not being used by lvm
        """
        name = "/dev/sda5"

        comt = "Physical Volume {0} does not exist".format(name)

        ret = {"name": name, "changes": {}, "result": True, "comment": comt}

        mock = MagicMock(side_effect=[False, True])
        with patch.dict(lvm.__salt__, {"lvm.pvdisplay": mock}):
            self.assertDictEqual(lvm.pv_absent(name), ret)

            comt = "Physical Volume {0} is set to be removed".format(name)
            ret.update({"comment": comt, "result": None})
            with patch.dict(lvm.__opts__, {"test": True}):
                self.assertDictEqual(lvm.pv_absent(name), ret)

    # 'vg_present' function tests: 1

    def test_vg_present(self):
        """
        Test to create an LVM volume group
        """
        name = "/dev/sda5"

        comt = "Failed to create Volume Group {0}".format(name)

        ret = {"name": name, "changes": {}, "result": False, "comment": comt}

        mock = MagicMock(return_value=False)
        with patch.dict(lvm.__salt__, {"lvm.vgdisplay": mock, "lvm.vgcreate": mock}):
            with patch.dict(lvm.__opts__, {"test": False}):
                self.assertDictEqual(lvm.vg_present(name), ret)

            comt = "Volume Group {0} is set to be created".format(name)
            ret.update({"comment": comt, "result": None})
            with patch.dict(lvm.__opts__, {"test": True}):
                self.assertDictEqual(lvm.vg_present(name), ret)

    # 'vg_absent' function tests: 1

    def test_vg_absent(self):
        """
        Test to remove an LVM volume group
        """
        name = "/dev/sda5"

        comt = "Volume Group {0} already absent".format(name)

        ret = {"name": name, "changes": {}, "result": True, "comment": comt}

        mock = MagicMock(side_effect=[False, True])
        with patch.dict(lvm.__salt__, {"lvm.vgdisplay": mock}):
            self.assertDictEqual(lvm.vg_absent(name), ret)

            comt = "Volume Group {0} is set to be removed".format(name)
            ret.update({"comment": comt, "result": None})
            with patch.dict(lvm.__opts__, {"test": True}):
                self.assertDictEqual(lvm.vg_absent(name), ret)

    # 'lv_present' function tests: 1

    def test_lv_present(self):
        """
        Test to create a new logical volume
        """
        name = "/dev/sda5"

        comt = "Logical Volume {0} already present".format(name)

        ret = {"name": name, "changes": {}, "result": True, "comment": comt}

        mock = MagicMock(side_effect=[True, False])
        with patch.dict(lvm.__salt__, {"lvm.lvdisplay": mock}):
            self.assertDictEqual(lvm.lv_present(name), ret)

            comt = "Logical Volume {0} is set to be created".format(name)
            ret.update({"comment": comt, "result": None})
            with patch.dict(lvm.__opts__, {"test": True}):
                self.assertDictEqual(lvm.lv_present(name), ret)

    def test_lv_present_with_force(self):
        """
        Test to create a new logical volume with force=True
        """
        name = "/dev/sda5"

        comt = "Logical Volume {0} already present".format(name)

        ret = {"name": name, "changes": {}, "result": True, "comment": comt}

        mock = MagicMock(side_effect=[True, False])
        with patch.dict(lvm.__salt__, {"lvm.lvdisplay": mock}):
            self.assertDictEqual(lvm.lv_present(name, force=True), ret)

            comt = "Logical Volume {0} is set to be created".format(name)
            ret.update({"comment": comt, "result": None})
            with patch.dict(lvm.__opts__, {"test": True}):
                self.assertDictEqual(lvm.lv_present(name, force=True), ret)

    # 'lv_absent' function tests: 1

    def test_lv_absent(self):
        """
        Test to remove a given existing logical volume
        from a named existing volume group
        """
        name = "/dev/sda5"

        comt = "Logical Volume {0} already absent".format(name)

        ret = {"name": name, "changes": {}, "result": True, "comment": comt}

        mock = MagicMock(side_effect=[False, True])
        with patch.dict(lvm.__salt__, {"lvm.lvdisplay": mock}):
            self.assertDictEqual(lvm.lv_absent(name), ret)

            comt = "Logical Volume {0} is set to be removed".format(name)
            ret.update({"comment": comt, "result": None})
            with patch.dict(lvm.__opts__, {"test": True}):
                self.assertDictEqual(lvm.lv_absent(name), ret)



# -*- coding: utf-8 -*-
"""
Base classes for napalm unit tests

:codeauthor: :email:`Anthony Shaw <anthonyshaw@apache.org>`
"""

from __future__ import absolute_import

from functools import wraps

TEST_INTERFACES = {
    "Management1": {
        "is_up": False,
        "is_enabled": False,
        "description": u"",
        "last_flapped": -1,
        "speed": 1000,
        "mac_address": u"dead:beef:dead",
    }
}

# Test data
TEST_FACTS = {
    "__opts__": {},
    "OPTIONAL_ARGS": {},
    "uptime": "Forever",
    "UP": True,
    "HOSTNAME": "test-device.com",
    "hostname": "test-device.com",
    "username": "admin",
    "os_version": "1.2.3",
    "model": "test_model",
    "serial_number": "123456",
    "vendor": "cisco",
    "interface_list": TEST_INTERFACES,
}

TEST_ENVIRONMENT = {"hot": "yes"}

TEST_COMMAND_RESPONSE = {"show run": "all the command output"}

TEST_TRACEROUTE_RESPONSE = {
    "success": {
        1: {
            "probes": {
                1: {
                    "rtt": 1.123,
                    "ip_address": u"206.223.116.21",
                    "host_name": u"eqixsj-google-gige.google.com",
                }
            }
        }
    }
}

TEST_PING_RESPONSE = {
    "success": {
        "probes_sent": 5,
        "packet_loss": 0,
        "rtt_min": 72.158,
        "rtt_max": 72.433,
        "rtt_avg": 72.268,
        "rtt_stddev": 0.094,
        "results": [{"ip_address": "1.1.1.1", "rtt": 72.248}],
    }
}

TEST_ARP_TABLE = [
    {
        "interface": "MgmtEth0/RSP0/CPU0/0",
        "mac": "5C:5E:AB:DA:3C:F0",
        "ip": "172.17.17.1",
        "age": 1454496274.84,
    }
]

TEST_IPADDRS = {"FastEthernet8": {"ipv4": {"10.66.43.169": {"prefix_length": 22}}}}

TEST_INTERFACES = {
    "Management1": {
        "is_up": False,
        "is_enabled": False,
        "description": u"",
        "last_flapped": -1,
        "speed": 1000,
        "mac_address": u"dead:beef:dead",
    }
}

TEST_LLDP_NEIGHBORS = {u"Ethernet2": [{"hostname": u"junos-unittest", "port": u"520"}]}

TEST_MAC_TABLE = [
    {
        "mac": "00:1C:58:29:4A:71",
        "interface": "Ethernet47",
        "vlan": 100,
        "static": False,
        "active": True,
        "moves": 1,
        "last_move": 1454417742.58,
    }
]

TEST_RUNNING_CONFIG = {"one": "two"}

TEST_OPTICS = {
    "et1": {
        "physical_channels": {
            "channel": [
                {
                    "index": 0,
                    "state": {
                        "input_power": {
                            "instant": 0.0,
                            "avg": 0.0,
                            "min": 0.0,
                            "max": 0.0,
                        },
                        "output_power": {
                            "instant": 0.0,
                            "avg": 0.0,
                            "min": 0.0,
                            "max": 0.0,
                        },
                        "laser_bias_current": {
                            "instant": 0.0,
                            "avg": 0.0,
                            "min": 0.0,
                            "max": 0.0,
                        },
                    },
                }
            ]
        }
    }
}

TEST_BGP_CONFIG = {"test": "value"}

TEST_BGP_NEIGHBORS = {
    "default": {
        8121: [
            {
                "up": True,
                "local_as": 13335,
                "remote_as": 8121,
                "local_address": u"172.101.76.1",
                "local_address_configured": True,
                "local_port": 179,
                "remote_address": u"192.247.78.0",
                "router_id": u"192.168.0.1",
                "remote_port": 58380,
                "multihop": False,
                "import_policy": u"4-NTT-TRANSIT-IN",
                "export_policy": u"4-NTT-TRANSIT-OUT",
                "input_messages": 123,
                "output_messages": 13,
                "input_updates": 123,
                "output_updates": 5,
                "messages_queued_out": 23,
                "connection_state": u"Established",
                "previous_connection_state": u"EstabSync",
                "last_event": u"RecvKeepAlive",
                "suppress_4byte_as": False,
                "local_as_prepend": False,
                "holdtime": 90,
                "configured_holdtime": 90,
                "keepalive": 30,
                "configured_keepalive": 30,
                "active_prefix_count": 132808,
                "received_prefix_count": 566739,
                "accepted_prefix_count": 566479,
                "suppressed_prefix_count": 0,
                "advertise_prefix_count": 0,
                "flap_count": 27,
            }
        ]
    }
}

TEST_TERM_CONFIG = {"result": True, "already_configured": False}

TEST_NTP_PEERS = {
    "192.168.0.1": 1,
    "172.17.17.1": 2,
    "172.17.17.2": 3,
    "2400:cb00:6:1024::c71b:840a": 4,
}

TEST_NTP_SERVERS = {
    "192.168.0.1": 1,
    "172.17.17.1": 2,
    "172.17.17.2": 3,
    "2400:cb00:6:1024::c71b:840a": 4,
}

TEST_NTP_STATS = [
    {
        "remote": u"188.114.101.4",
        "referenceid": u"188.114.100.1",
        "synchronized": True,
        "stratum": 4,
        "type": u"-",
        "when": u"107",
        "hostpoll": 256,
        "reachability": 377,
        "delay": 164.228,
        "offset": -13.866,
        "jitter": 2.695,
    }
]

TEST_PROBES_CONFIG = {
    "probe1": {
        "test1": {
            "probe_type": "icmp-ping",
            "target": "192.168.0.1",
            "source": "192.168.0.2",
            "probe_count": 13,
            "test_interval": 3,
        },
        "test2": {
            "probe_type": "http-ping",
            "target": "172.17.17.1",
            "source": "192.17.17.2",
            "probe_count": 5,
            "test_interval": 60,
        },
    }
}

TEST_PROBES_RESULTS = {
    "probe1": {
        "test1": {
            "last_test_min_delay": 63.120,
            "global_test_min_delay": 62.912,
            "current_test_avg_delay": 63.190,
            "global_test_max_delay": 177.349,
            "current_test_max_delay": 63.302,
            "global_test_avg_delay": 63.802,
            "last_test_avg_delay": 63.438,
            "last_test_max_delay": 65.356,
            "probe_type": "icmp-ping",
            "rtt": 63.138,
            "last_test_loss": 0,
            "round_trip_jitter": -59.0,
            "target": "192.168.0.1",
            "source": "192.168.0.2",
            "probe_count": 15,
            "current_test_min_delay": 63.138,
        },
        "test2": {
            "last_test_min_delay": 176.384,
            "global_test_min_delay": 169.226,
            "current_test_avg_delay": 177.098,
            "global_test_max_delay": 292.628,
            "current_test_max_delay": 180.055,
            "global_test_avg_delay": 177.959,
            "last_test_avg_delay": 177.178,
            "last_test_max_delay": 184.671,
            "probe_type": "icmp-ping",
            "rtt": 176.449,
            "last_test_loss": 0,
            "round_trip_jitter": -34.0,
            "target": "172.17.17.1",
            "source": "172.17.17.2",
            "probe_count": 15,
            "current_test_min_delay": 176.402,
        },
    }
}

TEST_ROUTE = {
    "172.16.0.0/25": [
        {
            "protocol": "BGP",
            "last_active": True,
            "current_active": True,
            "age": 1178693,
            "routing_table": "inet.0",
            "next_hop": "192.168.0.11",
            "outgoing_interface": "xe-1/1/1.100",
            "preference": 170,
            "selected_next_hop": False,
            "protocol_attributes": {
                "remote_as": 65001,
                "metric": 5,
                "local_as": 13335,
                "as_path": "",
                "remote_address": "192.168.0.11",
                "metric2": 0,
                "local_preference": 0,
                "communities": ["0:2", "no-export"],
                "preference2": -1,
            },
            "inactive_reason": "",
        },
        {
            "protocol": "BGP",
            "last_active": False,
            "current_active": False,
            "age": 2359429,
            "routing_table": "inet.0",
            "next_hop": "192.168.0.17",
            "outgoing_interface": "xe-1/1/1.100",
            "preference": 170,
            "selected_next_hop": True,
            "protocol_attributes": {
                "remote_as": 65001,
                "metric": 5,
                "local_as": 13335,
                "as_path": "",
                "remote_address": "192.168.0.17",
                "metric2": 0,
                "local_preference": 0,
                "communities": ["0:3", "no-export"],
                "preference2": -1,
            },
            "inactive_reason": "Not Best in its group - Router ID",
        },
    ]
}

TEST_SNMP_INFO = {"test_": "value"}

TEST_USERS = {
    "mircea": {
        "level": 15,
        "password": "$1$0P70xKPa$4jt5/10cBTckk6I/w/",
        "sshkeys": [
            "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQC4pFn+shPwTb2yELO4L7NtQrKOJXNeCl1je\
            l9STXVaGnRAnuc2PXl35vnWmcUq6YbUEcgUTRzzXfmelJKuVJTJIlMXii7h2xkbQp0YZIEs4P\
            8ipwnRBAxFfk/ZcDsdfsdfsdfsdN56ejk345jhk345jk345jk341p3A/9LIL7l6YewLBCwJj6\
            D+fWSJ0/YW+7oH17Fk2HH+tw0L5PcWLHkwA4t60iXn16qDbIk/ze6jv2hDGdCdz7oYQeCE55C\
            CHOHMJWYfN3jcL4s0qv8/u6Ka1FVkV7iMmro7ChThoV/5snI4Ljf2wKqgHH7TfNaCfpU0WvHA\
            nTs8zhOrGScSrtb mircea@master-roshi"
        ],
    }
}


class MockNapalmDevice(object):
    """Setup a mock device for our tests"""

    def get_facts(self):
        return TEST_FACTS

    def get_environment(self):
        return TEST_ENVIRONMENT

    def get_arp_table(self):
        return TEST_ARP_TABLE

    def get(self, key, default=None, *args, **kwargs):
        try:
            if key == "DRIVER":
                return self
            return TEST_FACTS[key]
        except KeyError:
            return default

    def cli(self, commands, *args, **kwargs):
        assert commands[0] == "show run"
        return TEST_COMMAND_RESPONSE

    def traceroute(self, destination, **kwargs):
        assert destination == "destination.com"
        return TEST_TRACEROUTE_RESPONSE

    def ping(self, destination, **kwargs):
        assert destination == "destination.com"
        return TEST_PING_RESPONSE

    def get_config(self, retrieve="all"):
        assert retrieve == "running"
        return TEST_RUNNING_CONFIG

    def get_interfaces_ip(self, **kwargs):
        return TEST_IPADDRS

    def get_interfaces(self, **kwargs):
        return TEST_INTERFACES

    def get_lldp_neighbors_detail(self, **kwargs):
        return TEST_LLDP_NEIGHBORS

    def get_mac_address_table(self, **kwargs):
        return TEST_MAC_TABLE

    def get_optics(self, **kwargs):
        return TEST_OPTICS

    def load_merge_candidate(self, filename=None, config=None):
        assert config == "new config"
        return TEST_RUNNING_CONFIG

    def load_replace_candidate(self, filename=None, config=None):
        assert config == "new config"
        return TEST_RUNNING_CONFIG

    def commit_config(self, **kwargs):
        return TEST_RUNNING_CONFIG

    def discard_config(self, **kwargs):
        return TEST_RUNNING_CONFIG

    def compare_config(self, **kwargs):
        return TEST_RUNNING_CONFIG

    def rollback(self, **kwargs):
        return TEST_RUNNING_CONFIG

    def get_bgp_config(self, **kwargs):
        return TEST_BGP_CONFIG

    def get_bgp_neighbors_detail(self, neighbor_address=None, **kwargs):
        assert neighbor_address is None or "test_address"
        return TEST_BGP_NEIGHBORS

    def get_ntp_peers(self, **kwargs):
        return TEST_NTP_PEERS

    def get_ntp_servers(self, **kwargs):
        return TEST_NTP_SERVERS

    def get_ntp_stats(self, **kwargs):
        return TEST_NTP_STATS

    def get_probes_config(self, **kwargs):
        return TEST_PROBES_CONFIG

    def get_probes_results(self, **kwargs):
        return TEST_PROBES_RESULTS

    def get_route_to(self, destination, protocol=None, **kwargs):
        assert destination == "1.2.3.4"
        return TEST_ROUTE

    def get_snmp_information(self, **kwargs):
        return TEST_SNMP_INFO

    def get_users(self, **kwargs):
        return TEST_USERS


def mock_proxy_napalm_wrap(func):
    """
    The proper decorator checks for proxy minions. We don't care
    so just pass back to the origination function
    """

    @wraps(func)
    def func_wrapper(*args, **kwargs):
        func.__globals__["napalm_device"] = MockNapalmDevice()
        return func(*args, **kwargs)

    return func_wrapper


import salt.utils.napalm as napalm_utils  # NOQA

napalm_utils.proxy_napalm_wrap = mock_proxy_napalm_wrap  # NOQA


def true(name):
    return True


def random_hash(source, method):
    return 12346789


def join(*files):
    return True


def get_managed_file(*args, **kwargs):
    return "True"



# coding: utf-8

# Python libs
from __future__ import absolute_import

# Salt libs
import salt.beacons.smartos_vmadm as vmadm
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch

# Salt testing libs
from tests.support.unit import TestCase

# Mock Results
MOCK_CLEAN_STATE = {"first_run": True, "vms": []}
MOCK_VM_NONE = {}
MOCK_VM_ONE = {
    "00000000-0000-0000-0000-000000000001": {
        "state": "running",
        "alias": "vm1",
        "hostname": "vm1",
        "dns_domain": "example.org",
    },
}
MOCK_VM_TWO_STOPPED = {
    "00000000-0000-0000-0000-000000000001": {
        "state": "running",
        "alias": "vm1",
        "hostname": "vm1",
        "dns_domain": "example.org",
    },
    "00000000-0000-0000-0000-000000000002": {
        "state": "stopped",
        "alias": "vm2",
        "hostname": "vm2",
        "dns_domain": "example.org",
    },
}
MOCK_VM_TWO_STARTED = {
    "00000000-0000-0000-0000-000000000001": {
        "state": "running",
        "alias": "vm1",
        "hostname": "vm1",
        "dns_domain": "example.org",
    },
    "00000000-0000-0000-0000-000000000002": {
        "state": "running",
        "alias": "vm2",
        "hostname": "vm2",
        "dns_domain": "example.org",
    },
}


class SmartOSImgAdmBeaconTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test case for salt.beacons.vmadm
    """

    def setup_loader_modules(self):
        return {vmadm: {"__context__": {}, "__salt__": {}}}

    def test_non_list_config(self):
        """
        We only have minimal validation so we test that here
        """
        config = {}

        ret = vmadm.validate(config)

        self.assertEqual(ret, (False, "Configuration for vmadm beacon must be a list!"))

    def test_created_startup(self):
        """
        Test with one vm and startup_create_event
        """
        # NOTE: this should yield 1 created event + one state event
        with patch.dict(vmadm.VMADM_STATE, MOCK_CLEAN_STATE), patch.dict(
            vmadm.__salt__, {"vmadm.list": MagicMock(return_value=MOCK_VM_ONE)}
        ):

            config = [{"startup_create_event": True}]

            ret = vmadm.validate(config)
            self.assertEqual(ret, (True, "Valid beacon configuration"))

            ret = vmadm.beacon(config)
            res = [
                {
                    "alias": "vm1",
                    "tag": "created/00000000-0000-0000-0000-000000000001",
                    "hostname": "vm1",
                    "dns_domain": "example.org",
                },
                {
                    "alias": "vm1",
                    "tag": "running/00000000-0000-0000-0000-000000000001",
                    "hostname": "vm1",
                    "dns_domain": "example.org",
                },
            ]
            self.assertEqual(ret, res)

    def test_created_nostartup(self):
        """
        Test with one image and startup_import_event unset/false
        """
        # NOTE: this should yield 0 created event _ one state event
        with patch.dict(vmadm.VMADM_STATE, MOCK_CLEAN_STATE), patch.dict(
            vmadm.__salt__, {"vmadm.list": MagicMock(return_value=MOCK_VM_ONE)}
        ):

            config = []

            ret = vmadm.validate(config)
            self.assertEqual(ret, (True, "Valid beacon configuration"))

            ret = vmadm.beacon(config)
            res = [
                {
                    "alias": "vm1",
                    "tag": "running/00000000-0000-0000-0000-000000000001",
                    "hostname": "vm1",
                    "dns_domain": "example.org",
                }
            ]

            self.assertEqual(ret, res)

    def test_created(self):
        """
        Test with one vm, create a 2nd one
        """
        # NOTE: this should yield 1 created event + state event
        with patch.dict(vmadm.VMADM_STATE, MOCK_CLEAN_STATE), patch.dict(
            vmadm.__salt__,
            {"vmadm.list": MagicMock(side_effect=[MOCK_VM_ONE, MOCK_VM_TWO_STARTED])},
        ):

            config = []

            ret = vmadm.validate(config)
            self.assertEqual(ret, (True, "Valid beacon configuration"))

            # Initial pass (Initialized state and do not yield created events at startup)
            ret = vmadm.beacon(config)

            # Second pass (After create a new vm)
            ret = vmadm.beacon(config)
            res = [
                {
                    "alias": "vm2",
                    "tag": "created/00000000-0000-0000-0000-000000000002",
                    "hostname": "vm2",
                    "dns_domain": "example.org",
                },
                {
                    "alias": "vm2",
                    "tag": "running/00000000-0000-0000-0000-000000000002",
                    "hostname": "vm2",
                    "dns_domain": "example.org",
                },
            ]

            self.assertEqual(ret, res)

    def test_deleted(self):
        """
        Test with two vms and one gets destroyed
        """
        # NOTE: this should yield 1 destroyed event
        with patch.dict(vmadm.VMADM_STATE, MOCK_CLEAN_STATE), patch.dict(
            vmadm.__salt__,
            {"vmadm.list": MagicMock(side_effect=[MOCK_VM_TWO_STOPPED, MOCK_VM_ONE])},
        ):

            config = []

            ret = vmadm.validate(config)
            self.assertEqual(ret, (True, "Valid beacon configuration"))

            # Initial pass (Initialized state and do not yield created vms at startup)
            ret = vmadm.beacon(config)

            # Second pass (Destroying one vm)
            ret = vmadm.beacon(config)
            res = [
                {
                    "alias": "vm2",
                    "tag": "deleted/00000000-0000-0000-0000-000000000002",
                    "hostname": "vm2",
                    "dns_domain": "example.org",
                }
            ]

            self.assertEqual(ret, res)

    def test_complex(self):
        """
        Test with two vms, stop one, delete one
        """
        # NOTE: this should yield 1 delete and 2 import events
        with patch.dict(vmadm.VMADM_STATE, MOCK_CLEAN_STATE), patch.dict(
            vmadm.__salt__,
            {
                "vmadm.list": MagicMock(
                    side_effect=[MOCK_VM_TWO_STARTED, MOCK_VM_TWO_STOPPED, MOCK_VM_ONE]
                )
            },
        ):

            config = []

            ret = vmadm.validate(config)
            self.assertEqual(ret, (True, "Valid beacon configuration"))

            # Initial pass (Initialized state and do not yield created events at startup)
            ret = vmadm.beacon(config)

            # Second pass (Stop one vm)
            ret = vmadm.beacon(config)
            res = [
                {
                    "alias": "vm2",
                    "tag": "stopped/00000000-0000-0000-0000-000000000002",
                    "hostname": "vm2",
                    "dns_domain": "example.org",
                }
            ]

            self.assertEqual(ret, res)

            # Third pass (Delete one vm)
            ret = vmadm.beacon(config)
            res = [
                {
                    "alias": "vm2",
                    "tag": "deleted/00000000-0000-0000-0000-000000000002",
                    "hostname": "vm2",
                    "dns_domain": "example.org",
                }
            ]

            self.assertEqual(ret, res)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, print_function, unicode_literals

import re

import pytest
from tests.support.case import ModuleCase
from tests.support.helpers import slowTest


@pytest.mark.windows_whitelisted
class KeyModuleTest(ModuleCase):
    @slowTest
    def test_key_finger(self):
        """
        test key.finger to ensure we receive a valid fingerprint
        """
        out = self.run_function("key.finger")
        match = re.match("([0-9a-z]{2}:){15,}[0-9a-z]{2}$", out)
        self.assertTrue(match)

    @slowTest
    def test_key_finger_master(self):
        """
        test key.finger_master to ensure we receive a valid fingerprint
        """
        out = self.run_function("key.finger_master")
        match = re.match("([0-9a-z]{2}:){15,}[0-9a-z]{2}$", out)
        self.assertTrue(match)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.boto_iam_role as boto_iam_role

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class BotoIAMRoleTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.boto_iam_role
    """

    def setup_loader_modules(self):
        return {boto_iam_role: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure the IAM role exists.
        """
        name = "myrole"

        ret = {"name": name, "result": False, "changes": {}, "comment": ""}

        _desc_role = {
            "create_date": "2015-02-11T19:47:14Z",
            "role_id": "HIUHBIUBIBNKJNBKJ",
            "assume_role_policy_document": {
                "Version": "2008-10-17",
                "Statement": [
                    {
                        "Action": "sts:AssumeRole",
                        "Principal": {"Service": "ec2.amazonaws.com"},
                        "Effect": "Allow",
                    }
                ],
            },
            "role_name": "myfakerole",
            "path": "/",
            "arn": "arn:aws:iam::12345:role/myfakerole",
        }
        _desc_role2 = {
            "create_date": "2015-02-11T19:47:14Z",
            "role_id": "HIUHBIUBIBNKJNBKJ",
            "assume_role_policy_document": {
                "Version": "2008-10-17",
                "Statement": [
                    {
                        "Action": "sts:AssumeRole",
                        "Principal": {
                            "Service": [
                                "ec2.amazonaws.com",
                                "datapipeline.amazonaws.com",
                            ]
                        },
                        "Effect": "Allow",
                    }
                ],
            },
            "role_name": "myfakerole",
            "path": "/",
            "arn": "arn:aws:iam::12345:role/myfakerole",
        }
        mock_desc = MagicMock(
            side_effect=[False, _desc_role, _desc_role, _desc_role2, _desc_role]
        )
        _build_policy = {
            "Version": "2008-10-17",
            "Statement": [
                {
                    "Action": "sts:AssumeRole",
                    "Effect": "Allow",
                    "Principal": {"Service": "ec2.amazonaws.com"},
                }
            ],
        }
        mock_policy = MagicMock(return_value=_build_policy)
        mock_ipe = MagicMock(side_effect=[False, True, True, True])
        mock_pa = MagicMock(side_effect=[False, True, True, True])
        mock_bool = MagicMock(return_value=False)
        mock_lst = MagicMock(return_value=[])
        with patch.dict(
            boto_iam_role.__salt__,
            {
                "boto_iam.describe_role": mock_desc,
                "boto_iam.create_role": mock_bool,
                "boto_iam.build_policy": mock_policy,
                "boto_iam.update_assume_role_policy": mock_bool,
                "boto_iam.instance_profile_exists": mock_ipe,
                "boto_iam.list_attached_role_policies": mock_lst,
                "boto_iam.create_instance_profile": mock_bool,
                "boto_iam.profile_associated": mock_pa,
                "boto_iam.associate_profile_to_role": mock_bool,
                "boto_iam.list_role_policies": mock_lst,
            },
        ):
            with patch.dict(boto_iam_role.__opts__, {"test": False}):
                comt = " Failed to create {0} IAM role.".format(name)
                ret.update({"comment": comt})
                self.assertDictEqual(boto_iam_role.present(name), ret)

                comt = (
                    " myrole role present. " "Failed to create myrole instance profile."
                )
                ret.update({"comment": comt})
                self.assertDictEqual(boto_iam_role.present(name), ret)

                comt = (
                    " myrole role present.  Failed to associate myrole"
                    " instance profile with myrole role."
                )
                ret.update({"comment": comt})
                self.assertDictEqual(boto_iam_role.present(name), ret)

                comt = " myrole role present. Failed to update assume role" " policy."
                ret.update({"comment": comt})

                self.assertDictEqual(boto_iam_role.present(name), ret)
                comt = " myrole role present.    "
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(boto_iam_role.present(name), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure the IAM role is deleted.
        """
        name = "myrole"

        ret = {"name": name, "result": False, "changes": {}, "comment": ""}

        mock = MagicMock(
            side_effect=[
                ["mypolicy"],
                ["mypolicy"],
                False,
                True,
                False,
                False,
                True,
                False,
                False,
                False,
                True,
            ]
        )
        mock_bool = MagicMock(return_value=False)
        mock_lst = MagicMock(return_value=[])
        with patch.dict(
            boto_iam_role.__salt__,
            {
                "boto_iam.list_role_policies": mock,
                "boto_iam.delete_role_policy": mock_bool,
                "boto_iam.profile_associated": mock,
                "boto_iam.disassociate_profile_from_role": mock_bool,
                "boto_iam.instance_profile_exists": mock,
                "boto_iam.list_attached_role_policies": mock_lst,
                "boto_iam.delete_instance_profile": mock_bool,
                "boto_iam.role_exists": mock,
                "boto_iam.delete_role": mock_bool,
            },
        ):
            with patch.dict(boto_iam_role.__opts__, {"test": False}):
                comt = " Failed to add policy mypolicy to role myrole"
                ret.update(
                    {
                        "comment": comt,
                        "changes": {
                            "new": {"policies": ["mypolicy"]},
                            "old": {"policies": ["mypolicy"]},
                        },
                    }
                )
                self.assertDictEqual(boto_iam_role.absent(name), ret)

                comt = (
                    " No policies in role myrole."
                    " No attached policies in role myrole. Failed to disassociate "
                    "myrole instance profile from myrole role."
                )
                ret.update({"comment": comt, "changes": {}})
                self.assertDictEqual(boto_iam_role.absent(name), ret)

                comt = (
                    " No policies in role myrole."
                    " No attached policies in role myrole. "
                    " Failed to delete myrole instance profile."
                )
                ret.update({"comment": comt, "changes": {}})
                self.assertDictEqual(boto_iam_role.absent(name), ret)

                comt = (
                    " No policies in role myrole."
                    " No attached policies in role myrole.  myrole instance profile "
                    "does not exist. Failed to delete myrole iam role."
                )
                ret.update({"comment": comt, "changes": {}})
                self.assertDictEqual(boto_iam_role.absent(name), ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.pyrax_queues as pyrax_queues

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class PyraxQueuesTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.pyrax_queues
    """

    def setup_loader_modules(self):
        return {pyrax_queues: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure the RackSpace queue exists.
        """
        name = "myqueue"
        provider = "my-pyrax"

        ret = {"name": name, "changes": {}, "result": True, "comment": ""}

        mock_dct = MagicMock(
            side_effect=[
                {provider: {"salt": True}},
                {provider: {"salt": False}},
                {provider: {"salt": False}},
                False,
            ]
        )
        with patch.dict(pyrax_queues.__salt__, {"cloud.action": mock_dct}):
            comt = "{0} present.".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(pyrax_queues.present(name, provider), ret)

            with patch.dict(pyrax_queues.__opts__, {"test": True}):
                comt = "Rackspace queue myqueue is set to be created."
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(pyrax_queues.present(name, provider), ret)

            with patch.dict(pyrax_queues.__opts__, {"test": False}):
                comt = "Failed to create myqueue Rackspace queue."
                ret.update({"comment": comt, "result": False})
                self.assertDictEqual(pyrax_queues.present(name, provider), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure the named Rackspace queue is deleted.
        """
        name = "myqueue"
        provider = "my-pyrax"

        ret = {"name": name, "changes": {}, "result": True, "comment": ""}

        mock_dct = MagicMock(
            side_effect=[{provider: {"salt": False}}, {provider: {"salt": True}}]
        )
        with patch.dict(pyrax_queues.__salt__, {"cloud.action": mock_dct}):
            comt = "myqueue does not exist."
            ret.update({"comment": comt})
            self.assertDictEqual(pyrax_queues.absent(name, provider), ret)

            with patch.dict(pyrax_queues.__opts__, {"test": True}):
                comt = "Rackspace queue myqueue is set to be removed."
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(pyrax_queues.absent(name, provider), ret)



# -*- coding: utf-8 -*-

from __future__ import absolute_import

import logging

from tests.support.helpers import slowTest
from tests.support.unit import skipIf
from tests.unit.utils.scheduler.base import SchedulerTestsBase

try:
    import dateutil.parser

    HAS_DATEUTIL_PARSER = True
except ImportError:
    HAS_DATEUTIL_PARSER = False

log = logging.getLogger(__name__)


@skipIf(
    HAS_DATEUTIL_PARSER is False, "The 'dateutil.parser' library is not available",
)
class SchedulerSkipTest(SchedulerTestsBase):
    """
    Validate the pkg module
    """

    def setUp(self):
        super(SchedulerSkipTest, self).setUp()
        self.schedule.opts["loop_interval"] = 1

    def tearDown(self):
        self.schedule.reset()
        super(SchedulerSkipTest, self).tearDown()

    @slowTest
    def test_skip(self):
        """
        verify that scheduled job is skipped at the specified time
        """
        job_name = "test_skip"
        job = {
            "schedule": {
                job_name: {
                    "function": "test.ping",
                    "when": ["11/29/2017 4pm", "11/29/2017 5pm"],
                }
            }
        }

        # Add job to schedule
        self.schedule.opts.update(job)

        run_time = dateutil.parser.parse("11/29/2017 4:00pm")
        self.schedule.skip_job(
            job_name,
            {
                "time": run_time.strftime("%Y-%m-%dT%H:%M:%S"),
                "time_fmt": "%Y-%m-%dT%H:%M:%S",
            },
        )

        # Run 11/29/2017 at 4pm
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertNotIn("_last_run", ret)
        self.assertEqual(ret["_skip_reason"], "skip_explicit")
        self.assertEqual(ret["_skipped_time"], run_time)

        # Run 11/29/2017 at 5pm
        run_time = dateutil.parser.parse("11/29/2017 5:00pm")
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret["_last_run"], run_time)

    @slowTest
    def test_skip_during_range(self):
        """
        verify that scheduled job is skipped during the specified range
        """
        job_name = "test_skip_during_range"
        job = {
            "schedule": {
                job_name: {
                    "function": "test.ping",
                    "hours": "1",
                    "skip_during_range": {
                        "start": "11/29/2017 2pm",
                        "end": "11/29/2017 3pm",
                    },
                }
            }
        }

        # Add job to schedule
        self.schedule.opts.update(job)

        # eval at 1:30pm to prime.
        run_time = dateutil.parser.parse("11/29/2017 1:30pm")
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)

        # eval at 2:30pm, will not run during range.
        run_time = dateutil.parser.parse("11/29/2017 2:30pm")
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertNotIn("_last_run", ret)
        self.assertEqual(ret["_skip_reason"], "in_skip_range")
        self.assertEqual(ret["_skipped_time"], run_time)

        # eval at 3:30pm, will run.
        run_time = dateutil.parser.parse("11/29/2017 3:30pm")
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret["_last_run"], run_time)

    def test_skip_during_range_invalid_datestring(self):
        """
        verify that scheduled job is not not and returns the right error string
        """
        run_time = dateutil.parser.parse("11/29/2017 2:30pm")

        job_name1 = "skip_during_range_invalid_datestring1"
        job1 = {
            "schedule": {
                job_name1: {
                    "function": "test.ping",
                    "hours": "1",
                    "_next_fire_time": run_time,
                    "skip_during_range": {"start": "25pm", "end": "3pm"},
                }
            }
        }

        job_name2 = "skip_during_range_invalid_datestring2"
        job2 = {
            "schedule": {
                job_name2: {
                    "function": "test.ping",
                    "hours": "1",
                    "_next_fire_time": run_time,
                    "skip_during_range": {"start": "2pm", "end": "25pm"},
                }
            }
        }

        # Add job1 to schedule
        self.schedule.opts.update(job1)

        # Eval
        self.schedule.eval(now=run_time)

        # Check the first job
        ret = self.schedule.job_status(job_name1)
        _expected = (
            "Invalid date string for start in "
            "skip_during_range. Ignoring "
            "job {0}."
        ).format(job_name1)
        self.assertEqual(ret["_error"], _expected)

        # Clear out schedule
        self.schedule.opts["schedule"] = {}

        # Add job2 to schedule
        self.schedule.opts.update(job2)

        # Eval
        self.schedule.eval(now=run_time)

        # Check the second job
        ret = self.schedule.job_status(job_name2)
        _expected = (
            "Invalid date string for end in " "skip_during_range. Ignoring " "job {0}."
        ).format(job_name2)
        self.assertEqual(ret["_error"], _expected)

    @slowTest
    def test_skip_during_range_global(self):
        """
        verify that scheduled job is skipped during the specified range
        """
        job_name = "skip_during_range_global"
        job = {
            "schedule": {
                "skip_during_range": {
                    "start": "11/29/2017 2:00pm",
                    "end": "11/29/2017 3:00pm",
                },
                job_name: {"function": "test.ping", "hours": "1"},
            }
        }

        # Add job to schedule
        self.schedule.opts.update(job)

        # eval at 1:30pm to prime.
        run_time = dateutil.parser.parse("11/29/2017 1:30pm")
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)

        # eval at 2:30pm, will not run during range.
        run_time = dateutil.parser.parse("11/29/2017 2:30pm")
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertNotIn("_last_run", ret)
        self.assertEqual(ret["_skip_reason"], "in_skip_range")
        self.assertEqual(ret["_skipped_time"], run_time)

        # eval at 3:30pm, will run.
        run_time = dateutil.parser.parse("11/29/2017 3:30pm")
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret["_last_run"], run_time)

    @slowTest
    def test_run_after_skip_range(self):
        """
        verify that scheduled job is skipped during the specified range
        """
        job_name = "skip_run_after_skip_range"
        job = {
            "schedule": {
                job_name: {
                    "function": "test.ping",
                    "when": "11/29/2017 2:30pm",
                    "run_after_skip_range": True,
                    "skip_during_range": {
                        "start": "11/29/2017 2pm",
                        "end": "11/29/2017 3pm",
                    },
                }
            }
        }

        # Add job to schedule
        self.schedule.opts.update(job)

        # eval at 2:30pm, will not run during range.
        run_time = dateutil.parser.parse("11/29/2017 2:30pm")
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertNotIn("_last_run", ret)
        self.assertEqual(ret["_skip_reason"], "in_skip_range")
        self.assertEqual(ret["_skipped_time"], run_time)

        # eval at 3:00:01pm, will run.
        run_time = dateutil.parser.parse("11/29/2017 3:00:01pm")
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertEqual(ret["_last_run"], run_time)

    @slowTest
    def test_run_seconds_skip(self):
        """
        verify that scheduled job is skipped during the specified range
        """
        job_name = "run_seconds_skip"
        job = {"schedule": {job_name: {"function": "test.ping", "seconds": "10"}}}

        # Add job to schedule
        self.schedule.opts.update(job)

        # eval at 2:00pm, to prime the scheduler
        run_time = dateutil.parser.parse("11/29/2017 2:00pm")
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)

        # eval at 2:00:10pm
        run_time = dateutil.parser.parse("11/29/2017 2:00:10pm")
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)

        # Skip at 2:00:20pm
        run_time = dateutil.parser.parse("11/29/2017 2:00:20pm")
        self.schedule.skip_job(
            job_name,
            {
                "time": run_time.strftime("%Y-%m-%dT%H:%M:%S"),
                "time_fmt": "%Y-%m-%dT%H:%M:%S",
            },
        )
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertIn("_next_fire_time", ret)
        self.assertEqual(ret["_skip_reason"], "skip_explicit")
        self.assertEqual(ret["_skipped_time"], run_time)

        # Run at 2:00:30pm
        run_time = dateutil.parser.parse("11/29/2017 2:00:30pm")
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status(job_name)
        self.assertIn("_last_run", ret)



# -*- coding: utf-8 -*-
"""
    tests.integration.proxy.conftest
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Proxy related fixtures
"""
from __future__ import absolute_import, unicode_literals

import logging
import os

import pytest
import salt.utils.files
from salt.serializers import yaml
from tests.support.runtests import RUNTIME_VARS

log = logging.getLogger(__name__)


@pytest.fixture(scope="package", autouse=True)
def salt_proxy(request, salt_factories, salt_master):
    proxy_minion_id = "proxytest"
    root_dir = salt_factories._get_root_dir_for_daemon(proxy_minion_id)
    conf_dir = root_dir.join("conf").ensure(dir=True)
    RUNTIME_VARS.TMP_PROXY_CONF_DIR = conf_dir.strpath

    with salt.utils.files.fopen(os.path.join(RUNTIME_VARS.CONF_DIR, "proxy")) as rfh:
        config_defaults = yaml.deserialize(rfh.read())

    config_defaults["hosts.file"] = os.path.join(RUNTIME_VARS.TMP, "hosts")
    config_defaults["aliases.file"] = os.path.join(RUNTIME_VARS.TMP, "aliases")
    config_defaults["transport"] = request.config.getoption("--transport")
    config_defaults["root_dir"] = root_dir
    yield salt_factories.spawn_proxy_minion(
        request, proxy_minion_id, master_id="master", config_defaults=config_defaults
    )

    proxy_key_file = os.path.join(
        salt_master.config["pki_dir"], "minions", proxy_minion_id
    )
    log.debug("Proxy minion %r KEY FILE: %s", proxy_minion_id, proxy_key_file)
    if os.path.exists(proxy_key_file):
        os.unlink(proxy_key_file)
    else:
        log.warning("The proxy minion key was not found at %s", proxy_key_file)
